@@ -1,6 +1,6 @@
 ;;; ess-cust.el --- Customize variables for ESS
 
-;; Copyright (C) 1997--2004 A.J. Rossini, Rich M. Heiberger, Martin
+;; Copyright (C) 1997--2005 A.J. Rossini, Rich M. Heiberger, Martin
 ;;	Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
 ;; Original Author: A.J. Rossini <rossini@u.washington.edu>
@@ -55,7 +55,7 @@
   :group 'local)
 
 (defgroup ess-edit nil
-  "ESS: editing behavior."
+  "ESS: editing behavior, including coments/indentation."
   :group 'ess
   :prefix "ess-")
 
@@ -126,7 +126,7 @@
 
 ;; Variables (not user-changeable)
 
-(defvar ess-version "5.2.3"
+(defvar ess-version "5.2.6"
   "Version of ESS currently loaded.")
 
 (defvar no-doc
@@ -258,7 +258,7 @@ If this is a relative file name, it is relative to `ess-history-directory'."
 
 (defcustom ess-S-assign " <- "
   "*String to be used for left assignment in all S dialects.
- Currently only activated by \\[ess-add-MM-keys]."
+ Used by \\[ess-smart-underscore]."
   :group 'ess-S
   :type 'string)
 
@@ -343,61 +343,86 @@ bracket."
 
 ;;;*;;; Indentation parameters
 
-(defvar ess-auto-newline nil
+(defcustom ess-auto-newline nil
   "*Non-nil means automatically newline before and after braces
-inserted in S code.")
+inserted in S code."
+  :type 'boolean
+  :group 'ess-edit)
 
-(defvar ess-tab-always-indent t
+(defcustom ess-tab-always-indent t
   "*Non-nil means TAB in S mode should always reindent the current line,
-regardless of where in the line point is when the TAB command is used.")
+regardless of where in the line point is when the TAB command is used."
+  :type 'boolean
+  :group 'ess-edit)
 
-(defvar ess-indent-level 2
-  "*Indentation of S statements with respect to containing block.")
+(defcustom ess-indent-level 2
+  "*Indentation of S statements with respect to containing block."
+  :type 'integer
+  :group 'ess-edit)
 
-(defvar ess-brace-imaginary-offset 0
-  "*Imagined indentation of an open brace following a statement.")
+(defcustom ess-brace-imaginary-offset 0
+  "*Imagined indentation of an open brace following a statement."
+  :type 'integer
+  :group 'ess-edit)
 
-(defvar ess-brace-offset 0
+(defcustom ess-brace-offset 0
   "*Extra indentation for open braces.
-Compares with other text in same context.")
+Compares with other text in same context."
+  :type 'integer
+  :group 'ess-edit)
 
-(defvar ess-continued-statement-offset 2
-  "*Extra indent for lines not starting new statements.")
+(defcustom ess-continued-statement-offset 2
+  "*Extra indent for lines not starting new statements."
+  :type 'integer
+  :group 'ess-edit)
 
-(defvar ess-continued-brace-offset 0
+(defcustom ess-continued-brace-offset 0
   "*Extra indent for substatements that start with open-braces.
-This is in addition to ess-continued-statement-offset.")
+This is in addition to ess-continued-statement-offset."
+  :type 'integer
+  :group 'ess-edit)
 
-(defvar ess-arg-function-offset 2
+(defcustom ess-arg-function-offset 2
   "*Extra indent for internal substatements of function `foo' that called
 in `arg=foo(...)' form.
-If not number, the statements are indented at open-parenthesis following foo.")
+If not number, the statements are indented at open-parenthesis following foo."
+  :type 'integer
+  :group 'ess-edit)
 
 ;;added rmh 2Nov97 at request of Terry Therneau
-(defvar ess-close-brace-offset 0
-  "*Extra indentation for closing braces.")
+(defcustom ess-close-brace-offset 0
+  "*Extra indentation for closing braces."
+  :type 'integer
+  :group 'ess-edit)
 
 ;;added rmh 2Nov97 at request of Terry Therneau
-(defvar ess-fancy-comments t
-  "*Non-nil means distiguish between #, ##, and ### for indentation.")
+(defcustom ess-fancy-comments t
+  "*Non-nil means distiguish between #, ##, and ### for indentation."
+  :type 'boolean
+  :group 'ess-edit)
 
 
 ;; PeterDalgaard, 1Apr97 :
 ;;The default ess-else-offset should be 0, not 2 IMHO (try looking at
 ;;the ls() function, for instance).  Was 2.
-(defvar ess-else-offset 0
-  "*Extra indent for `else' lines.")
+(defcustom ess-else-offset 0
+  "*Extra indent for `else' lines."
+  :type 'integer
+  :group 'ess-edit)
 
-(defvar ess-expression-offset 4
+(defcustom ess-expression-offset 4
   "*Extra indent for internal substatements of `expression' that specified
 in `obj <- expression(...)' form.
 If not number, the statements are indented at open-parenthesis following
-`expression'.")
+`expression'."
+  :type 'integer
+  :group 'ess-edit)
 
 ;;;*;;; Editing styles
 
 ;;; **FIXME**  The following NEEDS to be customized.
-
+;; SJE: I disagree; this variable should not be customized; individual vars,
+;; such as ess-indent-level are already customizable.
 (defvar ess-default-style-list
   (list 'DEFAULT
 	(cons 'ess-indent-level ess-indent-level)
@@ -457,10 +482,24 @@ If not number, the statements are indented at open-parenthesis following
 	       (ess-expression-offset . 4)
 	       (ess-else-offset . 0)
 	       (ess-close-brace-offset . 2))))
-  "Predefined formatting styles for ess code")
+  "Predefined formatting styles for ESS code.
+Values for all groups, except DEFAULT, are fixed.
+To change the value of variables in the DEFAULT group, change
+the corresponding variables, e.g. `ess-indent-level'.
+The default style in use is controlled by `ess-default-style'.")
 
-(defvar ess-default-style 'DEFAULT
-  "*The default value of `ess-style'.")
+(defcustom ess-default-style 'DEFAULT
+  "*The default value of `ess-style'.
+See the variable `ess-style-alist' for how these groups (DEFAULT,
+GNU, BSD, ...) map onto different settings for variables."
+  :type '(choice (const DEFAULT)
+		 (const GNU)
+		 (const BSD)
+		 (const K&R)
+		 (const C++)
+		 (const :tag "Common R" :value 'RRR)
+		 (const CLB))
+  :group 'ess-edit)
 
 (defvar ess-style ess-default-style
   "*The buffer specific ESS indentation style.")
@@ -589,20 +628,28 @@ Good for evaluating ESS code."
 ;; FIXME : This is just for the S dialects;  need to define this for others,
 ;; -----
 ;;  {however  "XLS-mode" should just use standard lisp "beginning of function"}
-(defcustom ess-function-pattern
+
+(defcustom ess-R-function-pattern
+  (concat
+   "\\(\\(" ; EITHER
+   "\\s\"" ; quote
+   "\\(\\sw\\|\\s_\\)+\\(<-\\)?" ; symbol (replacement?)
+   "\\s\"" ; quote
+   "\\)\\|\\(" ; OR
+   "\\(^\\|[ ]\\)" ; beginning of name
+   "\\(\\sw\\|\\s_\\)+" ; symbol
+   "\\)\\)" ; END EITHER OR
+   "\\s-*\\(<-\\|=\\)" ; whitespace, assign, whitespace/nl
+   "\\(\\(\\s-\\|\n\\)*\\s<.*\\s>\\)*" ; whitespace, comment
+   "\\(\\s-\\|\n\\)*function\\s-*(" ; whitespace, function keyword, parenthesis
+   )
+  "The regular expression for matching the beginning of an R function."
+  :group 'ess
+  :type 'regexp)
+
+(defcustom ess-S-function-pattern
+  ;; the same as "R" - but allowing "_" in assign
   (concat
-;;-    "\\(" ; EITHER
-;;-    "\\s\"" ; quote
-;;-    "\\(\\sw\\|\\s_\\)+" ; symbol
-;;-    "\\s\"" ; quote
-;;-    "\\s-*\\(<-\\|_\\)\\(\\s-\\|\n\\)*" ; whitespace, assign, whitespace/nl
-;;-    "function\\s-*(" ; function keyword, parenthesis
-;;-    "\\)\\|\\(" ; OR
-;;-    "\\<\\(\\sw\\|\\s_\\)+" ; symbol
-;;-    "\\s-*\\(<-\\|_\\)\\(\\s-\\|\n\\)*" ; whitespace, assign, whitespace/nl
-;;-    "function\\s-*(" ; function keyword, parenthesis
-;;-    "\\)")
-   ;;----- new version by  "Stephen C. Pope" <scp@predict.com> :
    "\\(\\(" ; EITHER
    "\\s\"" ; quote
    "\\(\\sw\\|\\s_\\)+\\(<-\\)?" ; symbol (replacement?)
@@ -621,6 +668,7 @@ Good for evaluating ESS code."
   :group 'ess
   :type 'regexp)
 
+
 ;; Fixme: the following is just for S dialects :
 (defcustom ess-dumped-missing-re
   "\\(<-\nDumped\n\\'\\)\\|\\(<-\\(\\s \\|\n\\)*\\'\\)"
@@ -637,11 +685,6 @@ by `ess-function-template'."
   :type 'regexp)
 
 ;;;; This is tested for S dialects (actually only for R) -- be careful with it!
-;;(defcustom ess-help-arg-regexp "\\(['\"]?\\)\\([^,=)'\"]*\\)\\1"
-;;  "Reg(ular) Ex(pression) of help(.) arguments.  MUST: 2nd \\(.\\) = arg."
-;;  :group 'ess
-;;  :type  'regexp)
-
 (defvar ess-help-arg-regexp "\\(['\"]?\\)\\([^,=)'\"]*\\)\\1"
   "Reg(ular) Ex(pression) of help(.) arguments.  MUST: 2nd \\(.\\) = arg.")
 
@@ -688,6 +731,12 @@ been created using the variable `ess-r-versions'."
   :group 'ess-R
   :type 'string)
 
+(defcustom inferior-R-objects-command "objects(pos=%d, all.names=TRUE)\n"
+  "Format string for R command to get a list of objects at position %d.
+Used in e.g., \\[ess-execute-objects] or \\[ess-display-help-on-object]."
+  :group 'ess-command
+  :type 'string)
+
 (defcustom ess-r-versions '( "R-1" "R-2")
   "*List of partial strings for versions of R to access within ESS.
 Each string specifies the start of a filename.  If a filename
@@ -724,7 +773,9 @@ file."
     "c:/progra~1/Insightful/splus61net/server"
     "c:/progra~1/Insightful/splus6se"
     "c:/progra~1/Insightful/splus61se"
-    "c:/progra~1/Insightful/splus62se")
+    "c:/progra~1/Insightful/splus62se"
+    "c:/progra~1/Insightful/splus70"
+    "c:/progra~1/Insightful/splus71")
   "*List of possible values of the environment variable SHOME for recent
 releases of S-Plus.  These are the default locations for several
 current and recent releases of S-Plus.  If any of these pathnames
@@ -847,6 +898,12 @@ These arguments are currently passed only to S+6."
   :group 'ess-S
   :type 'string)
 
+(defcustom inferior-Splus-objects-command "objects(where=%d)\n"
+  "Format string for R command to get a list of objects at position %d.
+Used in e.g., \\[ess-execute-objects] or \\[ess-display-help-on-object]."
+  :group 'ess-command
+  :type 'string)
+
 (defcustom inferior-S+6-print-command "S_PRINT_COMMAND=gnuclientw.exe"
   "*Destination of print icon in S+6 for Windows Commands window."
   :group 'ess-S
@@ -891,6 +948,16 @@ version of the pathname."
 ;;		(setq SHOME (expand-file-name (concat pathname "/../..")))))))
 ;;      (setq-default inferior-Sqpe+6-SHOME-name SHOME)))
 
+(defcustom ess-S-quit-kill-buffers-p nil
+  "Controls whether S buffers should also be killed once a process is killed.
+This is used only when an iESS process is killed using C-c C-q.
+Possible values:
+nil - do not kill any S buffers associated with the process.
+t - kill S buffers associated with the process.
+ask - ask the user whether the S buffers should be killed."
+  :group 'ess-S
+  :type '(choice (const nil) (const t) (const ask)))
+
 (defcustom inferior-XLS-program-name "xlispstat"
   "*Program name for invoking an inferior ESS with \\[XLS]."
   :group 'ess-XLS
@@ -932,7 +999,7 @@ order for it to work right.  And Emacs is too smart for it."
 ;;; ess-editor and ess-pager,
 ;;; and inferior-ess-language-start
 ;;; apply in principle to the 15 files essd[s-]*.el
-;;; Several of the files (essd-sp4.el and essdsp6w.el) have more
+;;; Several of the files (essd-sp4.el and essd-sp6w.el) have more
 ;;; than one *-customize-alist.
 ;;; These variables are currently used only with the S language files for
 ;;; S S-Plus R.
@@ -963,28 +1030,22 @@ order for it to work right.  And Emacs is too smart for it."
     (if (equal system-type 'Apple-Macintosh) nil
       (if (featurep 'xemacs) "gnuclient -q" "emacsclient")))
   "*Pager called by S process with 'page()' command."
-;; Change made to provide a better help(function) experience with
-;; S+6 and xemacs
-;; gnuclient -q will open a buffer with an HTML help file
-;; you can view it with M-x browse-url-of-buffer
-:group 'ess
-:type 'string)
+  ;; Change made to provide a better help(function) experience with
+  ;; S+6 and xemacs
+  ;; gnuclient -q will open a buffer with an HTML help file
+  ;; you can view it with M-x browse-url-of-buffer
+  :group 'ess
+  :type 'string)
 
-(defcustom ess-editor nil
+(defvar ess-editor nil
   "*Editor by which the process sends information to an emacs buffer
-for editing and then to be returned to the process."
-  :group 'ess-proc
-  :type 'string)
+for editing and then to be returned to the process.")
 
-(defcustom ess-pager nil
-  "*Pager by which the process sends information to an emacs buffer."
-  :group 'ess-proc
-  :type '(choice (const nil) string))
+(defvar ess-pager nil
+  "*Pager by which the process sends information to an emacs buffer.")
 
-(defcustom inferior-ess-language-start nil
-  "*Initialization commands sent to the ESS process."
-  :group 'ess-proc
-  :type 'string)
+(defvar inferior-ess-language-start nil
+  "*Initialization commands sent to the ESS process.")
 
 (make-variable-buffer-local 'ess-editor)
 (make-variable-buffer-local 'ess-pager)
@@ -1018,20 +1079,20 @@ for editing and then to be returned to the process."
 ;;;;; user settable defaults
 (defvar inferior-S-program-name  inferior-S+3-program-name
   "*Program name for invoking an inferior ESS with S().")
+;;- (setq inferior-S-program
+;;-       (cond ((string= S-proc-prefix "S") "Splus")
+;;- 	    ((string= S-proc-prefix "R") "R")
+;;- 	    (t "S")
+;;- 	    ))
+;;(make-local-variable 'inferior-S-program)
 
 (defvar inferior-ess-program nil ;inferior-S-program-name
   "*Default program name for invoking inferior-ess().
 The other variables ...-program-name should be changed, for the
 corresponding program.")
 
-;;(make-local-variable 'inferior-S-program)
 (make-variable-buffer-local 'inferior-ess-program)
 (setq-default inferior-ess-program inferior-S-program-name)
-;;- (setq inferior-S-program
-;;-       (cond ((string= S-proc-prefix "S") "Splus")
-;;- 	    ((string= S-proc-prefix "R") "R")
-;;- 	    (t "S")
-;;- 	    ))
 
 
 (defvar inferior-ess-start-args ""
@@ -1099,31 +1160,6 @@ of Emacs until the code has been successfully evaluated."
   :group 'ess-proc
   :type 'boolean)
 
-;;; SJE - suggest this is not defcustom, looks like each mode sets this.
-;;; SJE - this still produces mismatch.
-(defcustom ess-save-lastvalue-command nil
-  "Default depends on the ESS language/dialect.
-
-This is the command to save the last value.  See S section for more details.
-
-Might have to:"
-;;(make-variable-buffer-local 'ess-save-lastvalue-command)
-;;(setq-default ess-save-lastvalue-command
-;;	      "assign(\"smode.lvsave\",.Last.value,frame=0)\n")
-  :group 'ess-command
-  :type 'string)
-
-;;; SJE - suggest this is not defcustom, looks like each mode sets this.
-;;; SJE - this still produces mismatch.
-(defcustom ess-retr-lastvalue-command nil
-  "Default is currently the S+ version."
-  :group 'ess-command
-  :type 'string)
-
-;;(make-variable-buffer-local 'ess-retr-lastvalue-command)
-;;(setq-default ess-retr-lastvalue-command
-;;	      ".Last.value <- get(\"smode.lvsave\",frame=0)\n")
-
  ; System variables
 
 ;;*;; Variables relating to multiple processes
@@ -1177,15 +1213,13 @@ This format string should use %s to substitute an object name."
 (make-variable-buffer-local 'inferior-ess-exit-command)
 (setq-default inferior-ess-exit-command "q()\n")
 
-(defcustom inferior-ess-search-list-command "search()\n"
+(defvar inferior-ess-search-list-command nil
   "`ess-language' command that prints out the search list;
 i.e. the list of directories and (recursive) objects that `ess-language' uses
 when it searches for objects.
 
-Really set in <ess-lang>-customize-alist in ess[dl]-*.el"
-  :group 'ess-command
-  :type 'string)
-
+Really set in <ess-lang>-customize-alist in ess[dl]-*.el")
+;; and hence made buffer-local via that scheme...
 
 (defcustom inferior-ess-names-command "names(%s)\n"
   "Format string for ESS command to extract names from an object.
@@ -1194,38 +1228,59 @@ Really set in <ess-lang>-customize-alist in ess[dl]-*.el"
   :group 'ess-command
   :type 'string)
 
-(defcustom inferior-ess-objects-command "ls()\n" ;; others: in (S) or (R)
-  "Format string for ESS command to get a list of objects at position %d.
-
-Don't include a newline at the end! Used in ess-execute-objects."
-  :group 'ess-command
-  :type 'string)
-
-(make-variable-buffer-local 'inferior-ess-objects-command)
-(setq-default inferior-ess-objects-command "ls()\n")
-
 (defcustom inferior-ess-get-prompt-command "options()$prompt\n"
   "Command to find the value of the current S prompt."
   :group 'ess-command
   :type 'string)
 
+(defvar ess-cmd-delay nil
+  "*Set to a positive number if ESS will include delays proportional to
+`ess-cmd-delay'  in some places. These delays are introduced to
+prevent timeouts in certain processes, such as completion.")
+(make-variable-buffer-local 'ess-cmd-delay)
+
+(defcustom ess-R-cmd-delay nil
+  "used to initialize `ess-cmd-delay'."
+  :group 'ess-command
+  :type '(choice (const nil) number))
+
+(defcustom ess-S+-cmd-delay 1.0
+  "used to initialize `ess-cmd-delay'."
+  :group 'ess-command
+  :type '(choice (const nil) number))
+
 ;;*;; Regular expressions
 
-(defcustom inferior-ess-prompt nil
+(defvar inferior-ess-prompt nil
   "The regular expression inferior ess mode uses for recognizing prompts.
  Constructed at run time from `inferior-ess-primary-prompt' and
-`inferior-ess-secondary-prompt'."
-  :group 'ess-proc
-  :type '(choice (const nil) string))
+`inferior-ess-secondary-prompt' within `inferior-ess-mode'.")
 
 (make-variable-buffer-local 'inferior-ess-prompt)
 
-(defcustom ess-change-sp-regexp
-  "\\(attach(\\([^)]\\|$\\)\\|detach(\\|collection(\\|library(\\|require(\\|source(\\)"
-  "The regexp for matching the ESS commands that change the search path."
+(defvar ess-change-sp-regexp ""
+  "The regexp for matching the S/R/.. commands that change the search path.")
+(make-variable-buffer-local 'ess-change-sp-regexp)
+
+(defcustom ess-S+-change-sp-regexp
+  "\\(attach(\\([^)]\\|$\\)\\|detach(\\|collection(\\|library(\\|module(\\|source(\\)"
+  "The regexp for matching the S-plus commands that change the search path."
   :group 'ess-proc
   :type 'regexp)
 
+(defcustom ess-S-change-sp-regexp
+  "\\(attach(\\([^)]\\|$\\)\\|detach(\\|library(\\|source(\\)"
+  "The regexp for matching the S commands that change the search path."
+  :group 'ess-proc
+  :type 'regexp)
+
+(defcustom ess-R-change-sp-regexp
+  "\\(attach(\\([^)]\\|$\\)\\|detach(\\|library(\\|require(\\|source(\\)"
+  "The regexp for matching the R commands that change the search path."
+  :group 'ess-proc
+  :type 'regexp)
+
+
 ;;*;; Process-dependent variables
 
 (defvar ess-search-list nil
@@ -1275,10 +1330,10 @@ session.")
 
 (defvar ess-mode-minibuffer-map nil)
 
-(defcustom ess-object-name-db-file "ess-namedb"
-  "File containing definitions for `ess-object-name-db'."
-  :group 'ess
-  :type 'file)
+;; SJE: Wed 29 Dec 2004 - following 3 ess-object* variables can be removed
+;; soon if no-one needs the completion code.
+(defvar ess-object-name-db-file "ess-namedb"
+  "File containing definitions for `ess-object-name-db'.")
 
 (defvar ess-object-name-db-file-loaded '()
   "List of programs whose name-db file has been loaded.")
@@ -1291,7 +1346,7 @@ See also function `ess-create-object-name-db'.")
 (make-variable-buffer-local 'ess-object-name-db)
 (setq-default ess-object-name-db nil)
 
-(defcustom ess-S-loop-timeout 1200000
+(defcustom ess-S-loop-timeout 2000000
   "Integer specifying how many loops ess-mode will wait for the prompt
 before signaling an error. Will be set to `ess-loop-timeout' in the S dialects'
 alists.  Increase this, if you have a fast(er) machine."
@@ -1318,101 +1373,131 @@ If nil, input is in the `font-lock-variable-name-face'."
   :group 'ess
   :type 'boolean)
 
-(defvar ess-mode-font-lock-keywords
-  '(("<<-\\|<-\\|->" ; 2004-01: dropped "_" -- TODO later: for function-name
-     . font-lock-reference-face)
-    ("\\<\\(TRUE\\|FALSE\\|T\\|F\\|NA\\|NULL\\|Inf\\|NaN\\)\\>"
-     . font-lock-type-face)
-    ("\\<\\(library\\|require\\|attach\\|detach\\|source\\)\\>"
-     . font-lock-reference-face)
-    ("\\<\\(while\\|for\\|in\\|repeat\\|if\\|else\\|switch\\|break\\|next\\|return\\|stop\\|warning\\|function\\)\\>"
-     . font-lock-keyword-face) ; TODO : drop "_" (here and below)
-    ("\\s\"?\\(\\(\\sw\\|\\s_\\)+\\(<-\\)?\\)\\s\"?\\s-*\\(<-\\|_\\|=\\)\\(\\s-\\|\n\\)*function"
-     1 font-lock-function-name-face t))
-  "Font-lock patterns used in `ess-mode' buffers.")
+(defvar ess-R-constants
+  '("TRUE" "FALSE" "NA" "NULL" "Inf" "NaN"))
 
-(defvar inferior-ess-font-lock-keywords
-  '(("<<-\\|<-\\|->"
-     . font-lock-reference-face)		; assign
-    ("^\\*\\*\\*.*\\*\\*\\*\\s *$"
-     . font-lock-comment-face) ; ess-mode msg
-    ("\\[,?[1-9][0-9]*,?\\]"
-     . font-lock-reference-face)	; Vector/matrix labels
-    ("\\<\\(TRUE\\|FALSE\\|T\\|F\\|NA\\|NULL\\|Inf\\|NaN\\)\\>"
-     . font-lock-type-face) ; keywords
-    ("\\<\\(library\\|attach\\|detach\\|source\\)\\>"
-     . font-lock-reference-face) ; modify search list or source new definitions
-    ("^Syntax error:"
-     . font-lock-reference-face);inferior-ess problems or errors
-    ("^Error:"
-     . font-lock-reference-face)
-    ("^Error in"
-     . font-lock-reference-face)
-    ("^Dumped"
-     . font-lock-reference-face)
-    ("^Warning messages:"
-     . font-lock-reference-face)
-    ("#"
-     . font-lock-comment-face) ; comment
-    ("^[^#]*#\\(.*$\\)"
-     (1 font-lock-comment-face keep t)) ; comments
-    ("\\s\"?\\(\\(\\sw\\|\\s_\\)+\\)\\s\"?\\s-*\\(<-\\|_\\)\\(\\s-\\|\n\\)*function"
-     1 font-lock-function-name-face t) ; function name
-    ("\\<\\(while\\|for\\|in\\|repeat\\|if\\|else\\|switch\\|break\\|next\\|return\\|stop\\|warning\\|function\\)\\>"
-     . font-lock-keyword-face) ; keywords
-    )
-  "Font-lock patterns used in inferior-ess-mode buffers.")
+(defvar ess-S-constants
+  (append ess-R-constants '("T" "F")))
 
-;; add-to-list() places keywords in front of the previous keywords
-;; input and prompt must appear in inferior-ess-font-lock-keywords
-;; in the order  prompt error, hence they appear here in the reverse
-;; order.
+;; first the common ones
+(defvar ess-S-modifyiers
+  '("library" "attach" "detach" "source" "module"))
+(defvar ess-R-modifyiers
+  '("library" "attach" "detach" "source" "require"))
 
-(if (not inferior-ess-font-lock-input)
-    (add-to-list 'inferior-ess-font-lock-keywords
-		 '("^[a-zA-Z0-9 ]*[>+]\\(.*$\\)"
-		   (1 font-lock-variable-name-face keep t));don't font-lock input
-		 ))
-(add-to-list 'inferior-ess-font-lock-keywords
-	     '("^[a-zA-Z0-9 ]*[>+]" . font-lock-keyword-face))	; prompt
+(defvar ess-R-keywords
+  '("while" "for" "in" "repeat" "if" "else" "switch" "break" "next"
+    "function" "return" "message" "warning" "stop"))
+(defvar ess-S-keywords
+  (append ess-R-keywords '("terminate")))
 
-(defvar ess-trans-font-lock-keywords
- inferior-ess-font-lock-keywords
- "Font-lock patterns used in `ess-transcript-mode' buffers.")
+(defvar ess-R-message-prefixes
+  '("Error:" "Error in"
+    "Warning:" "Warning in"
+    "Warning messages?"))
+(defvar ess-S-message-prefixes
+  (append ess-R-message-prefixes
+	  '("Syntax error:" "Dumped")))
 
 ;;
-;;(defvar ess-mode-font-lock-keywords
-;; '(("\\s\"?\\(\\(\\sw\\|\\s_\\)+\\)\\s\"?\\s-*\\(<-\\|_\\)\\(\\s-\\|\n\\)*function" 1 font-lock-function-name-face t)
-;;   ("<<?-\\|_" . font-lock-reference-face)
-;;   ("\\<\\(TRUE\\|FALSE\\|T\\|F\\|NA\\|NULL\\|Inf\\|NaN\\)\\>" . font-lock-type-face)
-;;   ("\\<\\(library\\|attach\\|detach\\|source\\)\\>" . font-lock-reference-face)
-;;   "\\<\\(while\\|for\\|in\\|repeat\\|if\\|else\\|switch\\|break\\|next\\|return\\|stop\\|warning\\|function\\)\\>")
-;; "Font-lock patterns used in ess-mode bufffers.")
-;;
-;;(defvar essd-S-inferior-font-lock-keywords
-;; '(("^[a-zA-Z0-9 ]*[>+]" . font-lock-keyword-face)	; prompt
-;;   ("^[a-zA-Z0-9 ]*[>+]\\(.*$\\)"
-;;    (1 font-lock-variable-name-face keep t)) ; input
-;;   ("<-\\|_" . font-lock-reference-face)		; assign
-;;   ("^\\*\\*\\\*.*\\*\\*\\*\\s *$" . font-lock-comment-face) ; ess-mode msg
-;;   ("\\[,?[1-9][0-9]*,?\\]" . font-lock-reference-face)	; Vector/matrix labels
-;;   ("^Syntax error:" . font-lock-reference-face) ; error message
-;;   ("^Error:" . font-lock-reference-face) ; error message
-;;   ("^Error in" . font-lock-reference-face) ; error message
-;;   ("^Dumped" . font-lock-reference-face) ; error message
-;;   ("^Warning:" . font-lock-reference-face) ; warning message
-;;   ("\\<\\(TRUE\\|FALSE\\|T\\|F\\|NA\\|NULL\\|Inf\\|NaN\\)\\>"
-;;    . font-lock-type-face)) ; keywords
-;; "Font-lock patterns for dialects of S, used in highlighting process
-;; buffers and transcripts.")
-;;
-;;(defvar inferior-ess-font-lock-keywords
-;;  essd-S-inferior-font-lock-keywords
-;; "Font-lock patterns used in inferior-ess-mode buffers.")
-;;
-;;(defvar ess-trans-font-lock-keywords
-;;  essd-S-inferior-font-lock-keywords
-;; "Font-lock patterns used in ess-transcript-mode buffers.")
+(defvar ess-R-assign-ops
+  '("<<-" "<-" "->") ; don't want "=" here which is not only for assign
+)
+(defvar ess-S-assign-ops
+  '("<<-" "<-" "_" "->") ; don't want "=" here which is not only for assign
+)
+
+(defvar ess-R-function-name-regexp
+  (concat "\\s\"?\\(\\(\\sw\\|\\s_\\)+"
+	  "\\(<-\\)?\\)\\s\"?\\s-*\\(<-\\)"
+	  "\\(\\s-\\|\n\\)*function")
+)
+(defvar ess-S-function-name-regexp
+  ess-R-function-name-regexp ; since "_" is deprecated for S-plus as well
+)
+
+
+(defvar ess-R-mode-font-lock-keywords
+  (list
+   (cons (regexp-opt ess-R-assign-ops)
+	 'font-lock-reference-face)	; assign
+   (cons (concat "\\<" (regexp-opt ess-R-constants 'enc-paren) "\\>")
+	 'font-lock-type-face)		; constants
+   (cons (concat "\\<" (regexp-opt ess-R-modifyiers 'enc-paren) "\\>")
+	 'font-lock-reference-face)	; modify search list or source
+					; new definitions
+   (cons (concat "\\<" (regexp-opt ess-R-keywords 'enc-paren) "\\>")
+	 'font-lock-keyword-face)	; keywords
+   (cons ess-R-function-name-regexp
+	 '(1 font-lock-function-name-face t))
+					; function name
+   )
+  "Font-lock patterns used in `R-mode' buffers.")
+
+(defvar ess-S-mode-font-lock-keywords
+  (list
+   (cons (regexp-opt ess-S-assign-ops)
+	 'font-lock-reference-face)	; assign
+   (cons (concat "\\<" (regexp-opt ess-S-constants 'enc-paren) "\\>")
+	 'font-lock-type-face)		; constants
+   (cons (concat "\\<" (regexp-opt ess-S-modifyiers 'enc-paren) "\\>")
+	 'font-lock-reference-face)	; modify search list or source
+					; new definitions
+   (cons (concat "\\<" (regexp-opt ess-S-keywords 'enc-paren) "\\>")
+	 'font-lock-keyword-face)	; keywords
+   (cons ess-S-function-name-regexp
+	 '(1 font-lock-function-name-face t))
+					; function name
+   )
+  "Font-lock patterns used in `S-mode' buffers.")
+
+
+
+
+(defvar inferior-ess-R-font-lock-keywords
+  (append
+   '(("^[a-zA-Z0-9 ]*[>+]" . font-lock-keyword-face)) ; "prompt" must be first
+
+   (if (not inferior-ess-font-lock-input) ;; don't font-lock input :
+       (list (cons "^[a-zA-Z0-9 ]*[>+]\\(.*$\\)"
+		   '(1 font-lock-variable-name-face keep t))) )
+
+   ess-R-mode-font-lock-keywords
+
+   (list
+    (cons "^\\*\\*\\*.*\\*\\*\\*\\s *$" 'font-lock-comment-face); ess-mode msg
+    (cons "\\[,?[1-9][0-9]*,?\\]" 'font-lock-reference-face);Vector/matrix labels
+    (cons (concat "^" (regexp-opt ess-R-message-prefixes 'enc-paren))
+	  'font-lock-reference-face) ; inferior-ess problems or errors
+    (cons "#" 	'font-lock-comment-face) ; comment
+    (cons "^[^#]*#\\(.*$\\)" '(1 font-lock-comment-face keep t)) ; comments
+    ))
+  "Font-lock patterns used in inferior-R-mode buffers.")
+
+(defvar inferior-ess-S-font-lock-keywords
+  (append
+   '(("^[a-zA-Z0-9 ]*[>+]" . font-lock-keyword-face)) ; "prompt" must be first
+
+   (if (not inferior-ess-font-lock-input) ;; don't font-lock input :
+       (list (cons "^[a-zA-Z0-9 ]*[>+]\\(.*$\\)"
+		   '(1 font-lock-variable-name-face keep t))) )
+
+   ess-S-mode-font-lock-keywords
+
+   (list
+    (cons "^\\*\\*\\*.*\\*\\*\\*\\s *$" 'font-lock-comment-face) ; ess-mode msg
+    (cons "\\[,?[1-9][0-9]*,?\\]" 'font-lock-reference-face);Vector/matrix labels
+    (cons (concat "^" (regexp-opt ess-S-message-prefixes 'enc-paren))
+	  'font-lock-reference-face) ; inferior-ess problems or errors
+    (cons "#" 'font-lock-comment-face)	; comment
+    (cons "^[^#]*#\\(.*$\\)" '(1 font-lock-comment-face keep t)) ; comments
+    ))
+  "Font-lock patterns used in inferior-S-mode buffers.")
+
+;; use the inferior-* ones directly in ess-trns.el
+;; (defvar ess-trans-font-lock-keywords
+;;   inferior-ess-font-lock-keywords
+;;   "Font-lock patterns used in `ess-transcript-mode' buffers.")
 
 
 ;;;*;;; ess-help variables
@@ -1443,10 +1528,6 @@ the variable `ess-help-own-frame' is non-nil."
   :group 'ess-help
   :type 'alist)
 
-(defconst ess-help-S-sec-regex "^[A-Z. ---]+:$"
-  "Reg(ular) Ex(pression) of section headers in help file")
-
-
 
  ; User changeable variables
 ;;;=====================================================
@@ -1479,7 +1560,7 @@ Choices are `separate-buffer', `s-process', `www'.  The latter uses
 
 ;;*;; Variables relating to ess-help-mode
 
-;;-- ess-help-S-.. and  ess-help-R-.. constants now  in   S.el (are used in ess-inf).
+;;-- ess-help-S-.. and  ess-help-R-.. : in  essl-s.el (are used in ess-inf).
 
 (defvar ess-help-sec-keys-alist nil
   "Alist of (key . string) pairs for use in section searching.")
@@ -1513,6 +1594,11 @@ Created for each process."
   :group 'ess-proc
   :type 'string)
 
+(defcustom ess-verbose nil
+  "if non-nil, write more information to `ess-dribble-buffer' than usual."
+  :group 'ess-proc
+  :type 'boolean)
+
 (defvar ess-dribble-buffer (generate-new-buffer "*ESS*")
   "Buffer for temporary use for setting default variable values.
 Used for recording status of the program, mainly for debugging.")
