@@ -172,7 +172,7 @@ Uses the variable `inferior-ess-help-command' for the actual help command."
 	      (if (not (listp nodocs))
 		  (setq nodocs (list PM (point-max))))
 	      (ess-write-to-dribble-buffer
-	       (format "(ess-help: error-buffer %s nodocs (%d %d)\n"
+	       (format "(ess-help: error-buffer '%s' nodocs (%d %d)\n"
 		       (buffer-name) (car nodocs) (cadr nodocs)))
 	      ;; Avoid using 'message here -- may be %'s in string
 	      ;;(princ (buffer-substring (car nodocs) (cadr nodocs)) t)
@@ -185,7 +185,7 @@ Uses the variable `inferior-ess-help-command' for the actual help command."
 	  ;; else : show it
 
 	  ;;dbg (ess-write-to-dribble-buffer
-	  ;;dbg	 (format "(ess-help %s before switch-to..\n" hb-name)
+	  ;;dbg	 (format "(ess-help '%s' before switch-to..\n" hb-name)
 	  (let ((special-display-regexps
 		 (if ess-help-own-frame '(".") nil))
 		(special-display-frame-alist ess-help-frame-alist)
@@ -227,15 +227,15 @@ Uses the variable `inferior-ess-help-command' for the actual help command."
 
 ;;; THIS WORKS!
 ;;(require 'w3)
-(defun ess-display-w3-help-on-object-other-window (object)
-  "Display R-documentation for OBJECT using W3"
-  (interactive "s Help on :")
-  (let* ((ess-help-url (concat ess-help-w3-url-prefix
-			       ess-help-w3-url-funs
-			       object
-			       ".html")))
+;(defun ess-display-w3-help-on-object-other-window (object)
+;  "Display R-documentation for OBJECT using W3"
+;  (interactive "s Help on :")
+;  (let* ((ess-help-url (concat ess-help-w3-url-prefix
+;			       ess-help-w3-url-funs
+;			       object
+;			       ".html")))
     ;;(w3-fetch-other-window ess-help-url)
-    ))
+;    ))
 
 
 ;;*;; Major mode definition
@@ -266,6 +266,8 @@ Uses the variable `inferior-ess-help-command' for the actual help command."
   (define-key ess-help-mode-map "\177" 'scroll-down) ; DEL
   (define-key ess-help-mode-map "s" ess-help-sec-map)
   (define-key ess-help-mode-map "h" 'ess-display-help-on-object)
+;; TODO: `electric mouse-2'
+;; (define-key ess-help-mode-map [mouse-2] 'ess-display-help-on-object)
   (define-key ess-help-mode-map "l" 'ess-eval-line-and-step)
   (define-key ess-help-mode-map "r" 'ess-eval-region-and-go)
   (define-key ess-help-mode-map "n" 'ess-skip-to-next-section)
@@ -344,7 +346,7 @@ Other keybindings are as follows:
 		    "Menu keymap for ess-help mode." ess-help-mode-menu)
   (easy-menu-add ess-help-mode-menu-map ess-help-mode-map)
 
-  (run-hooks ess-help-mode-hook))
+  (run-hooks 'ess-help-mode-hook))
 
 ;;*;; User commands defined in ESS help mode
 
