@@ -3,7 +3,7 @@
 ;; Copyright (C) 1989-1994 Bates, Kademan, Ritter and Smith
 ;; Copyright (C) 1997-1999 A.J. Rossini <rossini@u.washington.edu>,
 ;;	Martin Maechler <maechler@stat.math.ethz.ch>.
-;; Copyright (C) 2000--2004 A.J. Rossini, Rich M. Heiberger, Martin
+;; Copyright (C) 2000--2005 A.J. Rossini, Rich M. Heiberger, Martin
 ;;	Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
 ;; Original Author: David Smith <dsmith@stats.adelaide.edu.au>
@@ -48,6 +48,7 @@
 (autoload 'ess-beginning-of-function	    "ess-mode"  "(autoload)." t)
 (autoload 'ess-end-of-function		    "ess-mode"  "(autoload)." t)
 (autoload 'ess-extract-word-name	    "ess-utils" "(autoload)." t)
+(autoload 'ess-uniq-list		    "ess-utils" "(autoload)." t)
 
 (autoload 'ess-transcript-send-command-and-move "ess-trns" "(autoload)." t)
 
@@ -180,7 +181,7 @@ accompany the call for `inferior-ess-program'.
 	(setq startdir
 	      (if ess-ask-for-ess-directory
 		  (ess-get-directory (directory-file-name defdir))
-		  defdir))
+		defdir))
 	(setq buf (current-buffer))
 	(ess-write-to-dribble-buffer
 	 (format "(inferior-ess) Method #1 start=%s buf=%s\n" startdir buf)))
@@ -249,6 +250,22 @@ accompany the call for `inferior-ess-program'.
 	(ess-multi procname buf inferior-ess-start-args)))))
 
 
+(defvar inferior-ess-objects-command nil
+  "The language/dialect specific command for listing objects.
+It is initialized from the corresponding inferior-<lang>-objects-command
+and then made buffer local."); and the *-<lang>-* ones are customized!
+(make-variable-buffer-local 'inferior-ess-objects-command)
+
+(defvar ess-save-lastvalue-command nil
+  "The command to save the last value.  See S section for more details.
+Default depends on the ESS language/dialect and hence made buffer local")
+(make-variable-buffer-local 'ess-save-lastvalue-command)
+
+(defvar ess-retr-lastvalue-command nil
+  "The command to retrieve the last value.  See S section for more details.
+Default depends on the ESS language/dialect and hence made buffer local")
+(make-variable-buffer-local 'ess-retr-lastvalue-command)
+
 ;;; A note on multiple processes: the following variables
 ;;;	ess-local-process-name
 ;;;	ess-search-list
@@ -292,16 +309,6 @@ there is no process NAME)."
 	 (format "(ess-multi 0):  inf-ess-start-args=%s, comint-..echoes=%s\n"
 	       inf-ess-start-args comint-process-echoes))
 	(set-buffer buffer)
-	(setq-default inferior-ess-prompt
-		      ;; shouldn't be setq-default!  And I've
-		      ;; forgotten why!   (AJR)
-
-		      ;; Do not anchor to bol with `^'
-		      (concat "\\("
-			      inferior-ess-primary-prompt
-			      "\\|"
-			      inferior-ess-secondary-prompt
-			      "\\)"))
 	(inferior-ess-mode)
 	(ess-write-to-dribble-buffer
 	 (format "(ess-multi post inf-ess: start-args=%s, comint-echoes=%s\n"
@@ -338,11 +345,34 @@ there is no process NAME)."
 	(setq ess-sl-modtime-alist nil)
 	;; Get search list when needed
 	(setq ess-sp-change t)
+
+	;; Add the process filter to catch certain output.
+	(set-process-filter (get-process proc-name)
+			    'inferior-ess-output-filter)
+	
 	(run-hooks 'ess-post-run-hook))
-      (if inferior-ess-same-window
+      (if (and inferior-ess-same-window (not inferior-ess-own-frame))
 	  (switch-to-buffer (process-buffer (get-process proc-name)))
 	(pop-to-buffer (process-buffer (get-process proc-name)))))))
 
+(defun inferior-ess-output-filter (proc string)
+  "Standard output filter for the inferior ESS process.
+Ring Emacs bell if process output starts with an ASCII bell, and pass
+the rest to `comint-output-filter'.
+Taken from octave-mod.el."
+  (comint-output-filter proc (inferior-ess-strip-ctrl-g string)))
+
+(defun inferior-ess-strip-ctrl-g (string)
+  "Strip leading `^G' character.
+If STRING starts with a `^G', ring the Emacs bell and strip it.
+Depending on the value of `visible-bell', either the frame will
+flash or you'll hear a beep.  Taken from octave-mod.el."
+  (if (string-match "^\a" string)
+      (progn
+        (ding)
+        (setq string (substring string 1))))
+  string)
+
 
 (defun ess-process-sentinel (proc message)
   "Sentinel for use with ESS processes.
@@ -712,16 +742,19 @@ ordinary inferior process.  Alway nil on Unix machines."
 		   ess-local-process-name 'inferior-ess-ddeclient)
 		  (default-value 'inferior-ess-ddeclient)))))
 
-(defun ess-prompt-wait (proc &optional start-of-output)
+(defun ess-prompt-wait (proc &optional start-of-output sleep)
   "Wait for a prompt to appear at BOL of current buffer.
 PROC is the ESS process. Does not change point"
+  (if sleep (sleep-for sleep)); we sleep here, *and* wait below
   (if start-of-output nil (setq start-of-output (point-min)))
   (save-excursion
     (while (progn
 	     ;; get output if there is some ready
-	     (if (or (equal window-system 'w32) (equal window-system 'win32))
-		 (accept-process-output proc 0 1500) ; Microsoft is slow
-	       (accept-process-output proc 0 500))
+
+;; 	     (if (and ess-microsoft-p ess-ms-slow)
+;; 		 (accept-process-output proc 0 1500) ; Microsoft is slow
+	       (accept-process-output proc 0 500)
+;; 	       )
 	     (goto-char (marker-position (process-mark proc)))
 	     (beginning-of-line)
 	     (if (< (point) start-of-output) (goto-char start-of-output))
@@ -741,25 +774,31 @@ PROC is the ESS process. Does not change point"
 	  (if moving (goto-char (process-mark proc))))
       (set-buffer old-buffer))))
 
-(defun ess-command (com &optional buf)
+(defun ess-command (com &optional buf sleep)
   "Send the ESS process command COM and delete the output
 from the ESS process buffer.  If an optional second argument BUF exists
 save the output in that buffer. BUF is erased before use.
 COM should have a terminating newline.
-Guarantees that the value of .Last.value will be preserved."
-  ;; Use this function when you need to evaluate so S code, and the
+Guarantees that the value of .Last.value will be preserved.
+When optional third arg SLEEP is non-nil, `(sleep-for (* a SLEEP))'
+will be used in a few places where `a' is proportional to `ess-cmd-delay'."
+  ;; Use this function when you need to evaluate some S code, and the
   ;; result is needed immediately. Waits until the output is ready
   (let* ((sprocess (get-ess-process ess-current-process-name))
 	 sbuffer
-	 end-of-output
-	 oldpb
-	 oldpf
-	 oldpm)
+	 do-sleep end-of-output
+	 oldpb oldpf oldpm
+	 my-retr-cmd my-save-cmd)
     (if sprocess nil
       (error "Process %s is not running!" ess-current-process-name))
     (setq sbuffer (process-buffer sprocess))
     (save-excursion
       (set-buffer sbuffer)
+      (setq do-sleep		    ; only now when in sprocess buffer
+	    (progn
+	      (if sleep (if (numberp sleep) nil (setq sleep 1))); t means 1
+	      (and ess-cmd-delay sleep)))
+      (if do-sleep (setq sleep (* sleep ess-cmd-delay)))
       (save-excursion
 	(goto-char (marker-position (process-mark sprocess)))
 	(beginning-of-line)
@@ -770,42 +809,37 @@ Guarantees that the value of .Last.value will be preserved."
       (setq oldpf (process-filter sprocess))
       (setq oldpb (process-buffer sprocess))
       (setq oldpm (marker-position (process-mark sprocess)))
+      ;; need the buffer-local values in result buffer "buf":
+      (setq my-retr-cmd ess-retr-lastvalue-command)
+      (setq my-save-cmd ess-save-lastvalue-command)
       (unwind-protect
 	  (progn
 	    (set-process-buffer sprocess buf)
 	    (set-process-filter sprocess 'ordinary-insertion-filter)
-	    ;; Output is now going to BUF
+	    ;; Output is now going to BUF:
 	    (save-excursion
 	      (set-buffer buf)
 	      (erase-buffer)
 	      (set-marker (process-mark sprocess) (point-min))
-	      (process-send-string sprocess ess-save-lastvalue-command)
-	      (if (or (equal window-system 'w32) (equal window-system 'win32))
-		  (sleep-for 0.5))
-	      (ess-prompt-wait sprocess)
+	      (process-send-string sprocess my-save-cmd)
+
+	      (ess-prompt-wait sprocess nil (and do-sleep (* 0.05 sleep)))
 	      (erase-buffer)
 	      (process-send-string sprocess com)
-	      (if (or (equal window-system 'w32) (equal window-system 'win32))
-		  (sleep-for 4))	;this much time is needed for
-					;ess-create-object-name-db on PC
-	      (ess-prompt-wait sprocess)
-	      (if (or (equal window-system 'w32) (equal window-system 'win32))
-		  (sleep-for 0.5))
+
+	      ;; need time for ess-create-object-name-db on PC
+	      (ess-prompt-wait sprocess nil (and do-sleep (* 0.4 sleep)));MS: 4
+	      ;;(if do-sleep (sleep-for (* 0.0 sleep))); microsoft: 0.5
 	      (goto-char (point-max))
 	      (save-excursion
 		(beginning-of-line)	; so prompt will be deleted
 		(setq end-of-output (point)))
-	      (process-send-string sprocess ess-retr-lastvalue-command)
+	      (process-send-string sprocess my-retr-cmd)
 
-	      ;; For S+4
-	      (if (or (equal window-system 'w32) (equal window-system 'win32))
-		  (sleep-for 0.5))
-	      (ess-prompt-wait sprocess end-of-output)
+	      (ess-prompt-wait sprocess end-of-output
+			       (and do-sleep (* 0.05 sleep))); microsoft: 0.5)
 
-	      ;; Old version.
-	      ;;(ess-prompt-wait sprocess (point))
-
-	      ;; Get rid out output from last assin
+	      ;; Get rid out output from last assign
 	      (delete-region end-of-output (point-max))))
 	;; Restore old values for process filter
 	(set-process-buffer sprocess oldpb)
@@ -906,7 +940,7 @@ EOB is non-nil go to end of ESS process buffer after evaluation.  If optional
 	(insert-before-markers (concat "*** " invisibly " ***\n")))
     ;; dbg:
     ;; dbg (ess-write-to-dribble-buffer
-    ;; dbg  (format "(eval-visibly 2): text[%d]= %s\n" (length text) text))
+    ;; dbg  (format "(eval-visibly 2): text[%d]= '%s'\n" (length text) text))
     (while (or (setq txt-gt-0 (> (length text) 0))
 	       even-empty)
       (if even-empty (setq even-empty nil))
@@ -1138,7 +1172,7 @@ process buffer. Arg has same meaning as for `ess-eval-region'."
 	    (ess-check-modifications))))
     (let ((errbuffer (ess-create-temp-buffer ess-error-buffer-name))
 	  error-occurred nomessage)
-      (ess-command (format inferior-ess-load-command filename) errbuffer)
+      (ess-command (format inferior-ess-load-command filename) errbuffer);sleep ?
       (save-excursion
 	(set-buffer errbuffer)
 	(goto-char (point-max))
@@ -1327,6 +1361,20 @@ to continue it."
   ;;	   inferior-ess-prompt)
   (comint-mode)
   ;;
+
+  ;; SJE: is this the proper place for setting inferior-ess-prompt,
+  ;; rather than within ess-multi?  Tony - have you remembered yet
+
+  ;; about the setq-default, as I changed it back to setq.
+  (setq inferior-ess-prompt
+	;; shouldn't be setq-default!  And I've
+	;; forgotten why!   (AJR)
+	;; Do not anchor to bol with `^'
+	(concat "\\("
+		inferior-ess-primary-prompt
+		"\\|"
+		inferior-ess-secondary-prompt
+		"\\)"))
   (setq comint-prompt-regexp (concat "^" inferior-ess-prompt))
   (setq major-mode 'inferior-ess-mode)
   (setq mode-name "iESS") ;(concat "iESS:" ess-dialect))
@@ -1409,10 +1457,16 @@ to continue it."
 	   (current-buffer) comint-process-echoes comint-input-sender))
 
   ;; Completion support
-  (setq comint-dynamic-complete-functions
-	'(ess-complete-filename
-	  ess-complete-object-name
-	  comint-replace-by-expanded-history))
+  ;; SJE: comint-dynamic-complete-functions is regarded as a hook, rather
+  ;; than a regular variable.
+  (add-hook 'comint-dynamic-complete-functions
+	    'ess-complete-object-name nil 'local)
+  (add-hook 'comint-dynamic-complete-functions
+	     'comint-replace-by-expanded-history 'append 'local)
+  (add-hook 'comint-dynamic-complete-functions
+	    'ess-complete-filename 'append 'local)
+
+
 
   ;; (setq comint-completion-addsuffix nil) ; To avoid spaces after filenames
   ;; KH: next 2 lines solve.
@@ -1430,9 +1484,9 @@ to continue it."
   (make-local-variable 'paragraph-separate)
   (setq paragraph-separate "\^L")
 
-
-  (ess-load-object-name-db-file)
-  (sleep-for 0.5)
+  ;; SJE Tue 28 Dec 2004: do not attempt to load object name db.
+  ;; (ess-load-object-name-db-file)
+  ;; (sleep-for 0.5)
   (make-local-variable 'kill-buffer-hook)
   (add-hook 'kill-buffer-hook 'ess-kill-buffer-function)
   (run-hooks 'inferior-ess-mode-hook)
@@ -1470,7 +1524,7 @@ to continue it."
 
 (defun inferior-R-input-sender (proc string)
   ;; REALLY only for debugging: this S_L_O_W_S D_O_W_N   [here AND below]
-  ;;(ess-write-to-dribble-buffer (format "(inf..-R-..): string=%s; " string))
+  ;;(ess-write-to-dribble-buffer (format "(inf..-R-..): string='%s'; " string))
   ;; rmh: 2002-01-12 catch page() in R
   (let ((help-string (or (string-match inferior-R-1-input-help string)
 			 (string-match inferior-R-2-input-help string)))
@@ -1478,7 +1532,7 @@ to continue it."
     (if (or help-string page-string)
 	(let* ((string2 (match-string 2 string))
 	       (string2-rt (concat string2 ".rt")))
-	  ;; (ess-write-to-dribble-buffer (format " new string=%s\n" string2))
+	  ;; (ess-write-to-dribble-buffer (format " new string='%s'\n" string2))
 	  (beginning-of-line)
 	  (if (looking-at inferior-ess-primary-prompt)
 	      (progn
@@ -1636,7 +1690,7 @@ to the command if BUFF is not given.)"
       (let ((buff (ess-create-temp-buffer buff-name)))
 	(save-excursion
 	  (set-buffer buff)
-	  (ess-command the-command (get-buffer buff-name))
+	  (ess-command the-command (get-buffer buff-name));; sleep?
 	  (goto-char (point-min))
 	  (if message (insert message)
 	    (if buff nil
@@ -1650,25 +1704,48 @@ to the command if BUFF is not given.)"
 ;;;*;;; Quitting
 
 (defun ess-quit ()
-  "Issue an exiting command to the inferior process, and clean up."
+  "Issue an exiting command to the inferior process, additionally
+also running \\[ess-cleanup].  For R, runs \\[ess-quit-r], see there."
   (interactive)
+  (if (string-equal ess-dialect "R")
+      (ess-quit-r)
+    (ess-force-buffer-current "Process to quit: ")
+    (ess-make-buffer-current)
+    (let ((sprocess (get-ess-process ess-current-process-name)))
+      (if (not sprocess) (error "No ESS process running."))
+      (when (yes-or-no-p (format "Really quit ESS process %s? " sprocess))
+	(ess-cleanup)
+	(goto-char (marker-position (process-mark sprocess)))
+	(insert inferior-ess-exit-command)
+	(process-send-string sprocess inferior-ess-exit-command)
+	;;SJE - suggest no need to rename buffer upon exit.
+	;;(rename-buffer (concat (buffer-name) "-exited") t)
+	))))
+
+(defun ess-quit-r ()
+  "Issue an exiting command to an inferior R process, and optionally clean up.
+This version is for killing *R* processes; it asks the extra question
+regarding whether the workspace image should be saved."
   (ess-force-buffer-current "Process to quit: ")
   (ess-make-buffer-current)
-  ;;  Modified to handle R (rmh 2002 Mar 12)
-  (let ((sprocess (get-ess-process ess-current-process-name)))
+  (let (cmd
+;;Q 	response
+	(sprocess (get-ess-process ess-current-process-name)))
     (if (not sprocess) (error "No ESS process running."))
-    (if (yes-or-no-p (format "Really quit ESS process %s? " sprocess))
-	(progn   ;;;;previouslyl (save-excursion
-	  (ess-cleanup)
-	  (goto-char (marker-position (process-mark sprocess)))
-	  (insert inferior-ess-exit-command)
-	  (if (string-equal ess-dialect "R")
-	      (progn
-		(inferior-ess-send-input)
-		(setq inferior-ess-prompt inferior-ess-exit-prompt)
-		(inferior-ess-wait-for-prompt))
-	    (process-send-string sprocess inferior-ess-exit-command))
-	  (rename-buffer (concat (buffer-name) "-exited") t)))))
+;;Q     (setq response (completing-read "Save workspace image? "
+;;Q 				    '( ( "yes".1) ("no" . 1) ("cancel" . 1))
+;;Q 				    nil t))
+;;Q     (if (string-equal response "")
+;;Q 	(setq response "default")); which will ask again (in most situations)
+;;Q     (unless (string-equal response "cancel")
+      (ess-cleanup)
+;;Q   (setq cmd (format "q(\"%s\")\n" response))
+      (setq cmd "q()\n")
+      (goto-char (marker-position (process-mark sprocess)))
+      (process-send-string sprocess cmd)
+      ;;(rename-buffer (concat (buffer-name) "-exited") t)
+;;Q      )
+  ))
 
 (defun ess-abort ()
   "Kill the ESS process, without executing .Last or terminating devices.
@@ -1684,30 +1761,32 @@ If you want to finish your session, use \\[ess-quit] instead."
     (message "Good move.")))
 
 (defun ess-cleanup ()
-  "Kill (or offer to kill) all buffers associated with this ESS process.
+  "Possibly kill or offer to kill, depending on the value of
+`ess-S-quit-kill-buffers-p', all buffers associated with this ESS process.
 Leaves you in the ESS process buffer.  It's a good idea to run this
 before you quit.  It is run automatically by \\[ess-quit]."
   (interactive)
   (let ((the-procname (or (ess-make-buffer-current) ess-local-process-name)))
     (if the-procname nil
       (error "I don't know which ESS process to clean up after!"))
-    (if (y-or-n-p
-	 (format
-	  "Delete all buffers associated with process %s? "
-	  the-procname))
-	(progn
-	  (save-excursion
-	    (mapcar '(lambda (buf)
-		       (set-buffer buf)
-		       ;; Consider buffers for which
-		       ;; ess-local-process-name is the same as
-		       ;; the-procname
-		       (if (and (not (get-buffer-process buf))
-				ess-local-process-name
-				(equal ess-local-process-name
-				       the-procname))
-			   (kill-buffer buf)))
-		    (buffer-list)))))
+    (if
+	(or (eq ess-S-quit-kill-buffers-p t)
+	    (and
+	     (eq ess-S-quit-kill-buffers-p 'ask)
+	     (y-or-n-p
+	      (format
+	       "Delete all buffers associated with process %s? " the-procname))))
+	(save-excursion
+	  (mapcar '(lambda (buf)
+		     (set-buffer buf)
+		     ;; Consider buffers for which
+		     ;; ess-local-process-name is the same as
+		     ;; the-procname
+		     (if (and (not (get-buffer-process buf))
+			      ess-local-process-name
+			      (equal ess-local-process-name the-procname))
+			 (kill-buffer buf)))
+		  (buffer-list))))
     (ess-switch-to-ESS nil)))
 
 (defun ess-kill-buffer-function nil
@@ -1750,9 +1829,8 @@ completions are listed [__UNIMPLEMENTED__]."
 			(point))
 		    (set-syntax-table buffer-syntax)))
 	     (full-prefix (buffer-substring beg end))
-	     ;; See if we're indexing a list with `$'
 	     (pattern full-prefix)
-	     ;; components  ; WHY IS THIS HERE?
+	     ;; See if we're indexing a list with `$'
 	     (listname (if (string-match "\\(.+\\)\\$\\(\\(\\sw\\|\\s_\\)*\\)$"
 					 full-prefix)
 			   (progn
@@ -1763,9 +1841,25 @@ completions are listed [__UNIMPLEMENTED__]."
 						(match-end 2))))
 			     (substring full-prefix (match-beginning 1)
 					(match-end 1)))))
-	     (components (if listname (ess-object-names listname)
-			   (ess-get-object-list
-			    ess-current-process-name))))
+	     ;; are we trying to get a slot via `@' ?
+	     (classname (if (string-match "\\(.+\\)@\\(\\(\\sw\\|\\s_\\)*\\)$"
+					 full-prefix)
+			   (progn
+			     (setq pattern
+				   (if (not (match-beginning 2)) ""
+				     (substring full-prefix
+						(match-beginning 2)
+						(match-end 2))))
+			     (ess-write-to-dribble-buffer
+			      (format "(ess-C-O-Name : slots..) : patt=%s"
+				      pattern))
+			     (substring full-prefix (match-beginning 1)
+					(match-end 1)))))
+	     (components (if listname
+			     (ess-object-names listname)
+			   (if classname
+			       (ess-slot-names classname)
+			     (ess-get-object-list ess-current-process-name)))))
 	;; always return a non-nil value to prevent history expansions
 	(or (comint-dynamic-simple-complete  pattern components) 'none))))
 
@@ -1777,7 +1871,7 @@ completions are listed [__UNIMPLEMENTED__]."
 ;;;*;;; Support functions
 (defun ess-extract-onames-from-alist (alist posn &optional force)
   "Return the object names in position POSN of ALIST.
-ALIST is an alist like `ess-sl-modtime alist'. POSN should be 1 .. (length
+ALIST is an alist like `ess-sl-modtime-alist'. POSN should be in 1 .. (length
 ALIST).	 If optional third arg FORCE is t, the corresponding element
 of the search list is re-read. Otherwise it is only re-read if it's a
 directory and has been modified since it was last read."
@@ -1786,23 +1880,20 @@ directory and has been modified since it was last read."
 	 (timestamp (car (cdr entry)))
 	 (new-modtime (ess-dir-modtime dir)))
     ;; Refresh the object listing if necessary
-    (if (and (not force) (equal new-modtime timestamp)) nil
-      (setq timestamp new-modtime)
-      (setcdr (cdr entry) (ess-object-names dir posn)))
+    (if (or force (not (equal new-modtime timestamp)))
+	(setcdr (cdr entry) (ess-object-names dir posn)))
     (cdr (cdr entry))))
 
 (defun ess-dir-modtime (dir)
   "Return the last modtime if DIR is a directory, and nil otherwise."
-  ;; Attached dataframes return a modtime of nil.
   (and ess-filenames-map
-       (file-directory-p dir); was (string-match "^/" dir)
+       (file-directory-p dir)
        (nth 5 (file-attributes dir))))
 
 (defun ess-object-modtime (object)
   "Return the modtime of the S object OBJECT (a string).
 Searches along the search list for a file named OBJECT and returns its modtime
-Returns nil if that file cannot be found.
-Does NOT (yet) work for R or any non-S language!"
+Returns nil if that file cannot be found, i.e., for R or any non-S language!"
   (let ((path (ess-search-list))
 	result)
     (while (and (not result) path)
@@ -1820,52 +1911,69 @@ Does NOT (yet) work for R or any non-S language!"
 		(> (car (cdr mod1)) (car (cdr mod2)))))))
 
 (defun ess-get-object-list (name)
-  "Return an alist of current S object names associated with process NAME."
+  "Return a list of current S object names associated with process NAME,
+using `ess-object-list' if that is non-nil."
   (or ess-object-list
       (save-excursion
 	(set-buffer (process-buffer (get-ess-process name)))
 	(ess-make-buffer-current)
+	(ess-write-to-dribble-buffer (format "(get-object-list %s) .." name))
 	(if (or (not ess-sl-modtime-alist) ess-sp-change)
-	    (ess-get-modtime-list))
+	    (progn (ess-write-to-dribble-buffer "--> (ess-get-modtime-list)\n")
+		   (ess-get-modtime-list))
+	  ;;else
+	  (ess-write-to-dribble-buffer " using existing ess-*-alist\n")
+	  )
 	(let* ((alist ess-sl-modtime-alist)
 	       (i 2)
 	       (n (length alist))
 	       result)
-	  ;; Always force a re-read of position 1 if it's a database
-	  (setq result (ess-extract-onames-from-alist alist 1 t))
+	  ;; Always force a re-read of position 1 :
+	  (setq result (ess-extract-onames-from-alist alist 1 'force))
 	  ;; Re-read remaining directories if necessary.
 	  (while (<= i n)
 	    (setq result
 		  (append result
 			  (ess-extract-onames-from-alist alist i)))
 	    (setq i (1+ i)))
-	  (setq ess-object-list result)))))
+	  (setq ess-object-list (ess-uniq-list result))))))
 
 (defun ess-get-words-from-vector (command)
   "Evaluate the S command COMMAND, which returns a character vector.
 Return the elements of the result of COMMAND as an alist of strings.
 A newline is automatically added to COMMAND."
-  (let ((tbuffer (get-buffer-create " *ess-names-list*"))
+  (let ((tbuffer (get-buffer-create
+		  " *ess-names-list*")); initial space: disable-undo
 	names)
     (save-excursion
       (set-buffer tbuffer)
-      ;; guaranteed by the initial space:(buffer-disable-undo)
-      (ess-command command tbuffer)
+      (ess-command command tbuffer 'sleep)
       (goto-char (point-min))
+      (if ess-verbose (ess-write-to-dribble-buffer "ess-get-words-.. "))
       (if (not (looking-at "\\s-*\\[1\\]"))
-	  (setq names nil)
+	  (progn (if ess-verbose
+		     (ess-write-to-dribble-buffer "not seeing \"[1]\".. "))
+		 (setq names nil)
+	  )
 	(goto-char (point-max))
 	(while (re-search-backward "\"\\([^\"]*\\)\"" nil t)
 	  (setq names (cons (buffer-substring (match-beginning 1)
 					      (match-end 1)) names))))
-      (kill-buffer tbuffer))
-    ;;DBG: (ess-write-to-dribble-buffer (format " --> names %s\n" names))
+      ;;DBG, do *not* (i.e., comment):
+      (kill-buffer tbuffer)
+      )
+    (if ess-verbose
+	(ess-write-to-dribble-buffer
+	 (if (> (length names) 5)
+	     (format " |-> (length names)= %d\n" (length names))
+	   (format " |-> names= '%s'\n" names))))
     names))
 
 (defun ess-compiled-dir (dir)
   "Return non-nil if DIR is an S object directory with special files.
 I.e. if the filenames in DIR are not representative of the objects in DIR."
-  (or (file-exists-p (concat (file-name-as-directory dir) "__BIGIN"))
+  (or (file-exists-p (concat (file-name-as-directory dir) "___nonfile"))
+      (file-exists-p (concat (file-name-as-directory dir) "__BIGIN"))
       (file-exists-p (concat (file-name-as-directory dir) "___NONFI"))))
 
 (defun ess-object-names (obj &optional pos)
@@ -1873,29 +1981,62 @@ I.e. if the filenames in DIR are not representative of the objects in DIR."
 If OBJ is a directory name (begins with `/') returns a listing of that dir.
    This may use the search list position POS if necessary.
 If OBJ is an object name, returns result of S command names(OBJ).
-If OBJ is nil, POS must be supplied, and objects(POS) is returned.
+If OBJ is nil or not a directory, POS must be supplied, and objects(POS) is returned.
 In all cases, the value is an list of object names."
+
+;; FIXME: in both cases below, use the same fallback "objects(POS)" -- merge!
   (if (and obj (file-accessible-directory-p obj))
       ;; Check the pre-compiled object list in ess-object-name-db first
+
+      ;; FIXME: If used at all, ess-object-name-db should not only
+      ;; -----  be used in the directory case !!
       (or (cdr-safe (assoc obj ess-object-name-db))
 	  ;; Take a directory listing
-	  (and ess-filenames-map (not (ess-compiled-dir obj))
-	       (directory-files obj))
+	  (and ess-filenames-map
+	       ;; first try .Data subdirectory:
+	       ;;FIXME: move ".Data" or ``this function'' to essd-sp6.el etc:
+	       (let ((dir (concat (file-name-as-directory obj) ".Data")))
+		 (if (not (file-accessible-directory-p dir))
+		     (setq dir obj))
+		 (and (not (ess-compiled-dir dir))
+		      (directory-files dir))))
 	  ;; Get objects(pos) instead
-	  (ess-get-words-from-vector
-	   (format inferior-ess-objects-command pos ".*")))
-    ;; Want names(obj)
-    (or (and obj (ess-get-words-from-vector
-		  (format inferior-ess-names-command obj)))
+	  (and (or (ess-write-to-dribble-buffer
+		    (format "(ess-object-names ..): directory %s not used\n" obj))
+		   t)
+	       pos
+	       (ess-get-words-from-vector
+		(format inferior-ess-objects-command pos))))
+	  ;; "else" should really give an error!
+	  ;; would need  pos = which(obj = search())
+
+    ;; else
+    (or (and obj  ;; want names(obj)
+	     (or (ess-write-to-dribble-buffer
+		  (format "(ess-object-names obj=%s): no directory - trying names\n"
+			  obj))
+		 t)
+	     (ess-get-words-from-vector
+	      (format inferior-ess-names-command obj)))
+	(and nil; must return nil
+	     (ess-write-to-dribble-buffer
+	      (format "(ess-object-names obj=%s): no dir.; -> objects()\n" obj)))
 	;; get objects(pos)
 	(ess-get-words-from-vector
-	 (format inferior-ess-objects-command pos ".*"))))) ; s4 needs 2
+	 (format inferior-ess-objects-command pos))))) ; had 2nd arg ".*"
+					; s4 needs 2
 					; args, rest only need 1 ?
 					; changes needed to allow for
 					; pattern argument to
 					; .SmodeObs
 
+(defun ess-slot-names (obj)
+  "Return alist of S4 slot names of S4 object OBJ."
+  (ess-get-words-from-vector (format "slotNames(%s)\n" obj)))
 
+
+;;; SJE: Wed 29 Dec 2004 --- remove this function.
+;;; rmh: Wed 5 Jan 2005 --- bring it back for use on Windows
 (defun ess-create-object-name-db ()
   "Create a database of object names in standard S directories.  This
 database is saved in the file specified by `ess-object-name-db-file',
@@ -1994,22 +2135,19 @@ but by \\[ess-resynch], \\[ess-get-object-list], \\[ess-get-modtime-list],
 and (indirectly) by \\[ess-get-help-files-list]."
   (save-excursion
     (let ((result nil))
-      (set-buffer (get-ess-buffer ess-current-process-name))
-      (if (and ess-search-list (not ess-sp-change)); use cache
+      (set-buffer (get-ess-buffer ess-current-process-name));to get *its* local vars
+      (if (and ess-search-list (not ess-sp-change))
+	  ;; use cache:
 	  ess-search-list
+	;; else, re-compute:
 	(let ((tbuffer (get-buffer-create " *search-list*"))
 	      (homedir ess-directory)
+	      (my-search-cmd inferior-ess-search-list-command); from ess-buffer
 	      elt)
 	  (save-excursion
 	    (set-buffer tbuffer)
-	    ;; guaranteed by the initial space:(buffer-disable-undo)
-	    ;; dbg:
-;;-	    (ess-write-to-dribble-buffer
-;;-	     (format "(ess-search-list ..): tbuffer %s\n"
-;;-		     (buffer-name tbuffer)))
-
-	    ;;unnecessary; ess-command does (erase-buffer)
-	    (ess-command inferior-ess-search-list-command tbuffer)
+	    ;; guaranteed by the initial space in its name: (buffer-disable-undo)
+	    (ess-command my-search-cmd tbuffer 0.2); <- sleep; does (erase-buffer)
 	    (goto-char (point-min))
 	    (while (re-search-forward "\"\\([^\"]*\\)\"" nil t)
 	      (setq elt (buffer-substring (match-beginning 1) (match-end 1)))
@@ -2021,7 +2159,7 @@ and (indirectly) by \\[ess-get-help-files-list]."
 		    (setq elt (concat homedir elt)))
 		;;else
 		;;dbg
-;;-		(ess-write-to-dribble-buffer "not dir.\n")
+		;;-		(ess-write-to-dribble-buffer "not dir.\n")
 		)
 	      (setq result (append result (list elt))))
 	    (kill-buffer tbuffer)))
@@ -2033,7 +2171,8 @@ and (indirectly) by \\[ess-get-help-files-list]."
 ;;;  * key	       (directory or object name)
 ;;;  * modtime	       (list of 2 integers)
 ;;;  * name, name ...  (accessible objects in search list posn labeled by key)
-;;; It has the same number of elements and is in the same order as the
+;;; It is a buffer-local variable (belonging to e.g. *R*, *S+6*, .. etc)
+;;; and has the same number of elements and is in the same order as the
 ;;; S search list
 
 (defun ess-get-modtime-list ()
@@ -2060,14 +2199,19 @@ The result is stored in `ess-sl-modtime-alist'."
 			  ))))))
       (setq index (1+ index))
       (setq searchlist (cdr searchlist)))
+    ;;DBG:
+    (ess-write-to-dribble-buffer
+     (format "ess-get-modtime-list: new alist of length %d\n"
+	     (length newalist)));; todo : also give length of components!
+
     (setq ess-sl-modtime-alist newalist)))
 
 
 (defun ess-search-path-tracker (str)
   "Check if input STR changed the search path.
 This function monitors user input to the inferior ESS process so that
-Emacs can keep the variable `ess-search-list' up to date.
-'completing-read' uses this list indirectly when it prompts for help or
+Emacs can keep the variable `ess-search-list' up to date. `completing-read' in
+\\[ess-read-object-name] uses this list indirectly when it prompts for help or
 for an object to dump."
   (if (string-match ess-change-sp-regexp str)
       (setq ess-sp-change t)))
@@ -2083,9 +2227,8 @@ P-STRING is the prompt string."
 	 (prompt-string (if default
 			    (format "%s(default %s) " p-string default)
 			  p-string))
-	 (ess-object-list (mapcar 'list (ess-get-object-list
-					 ess-local-process-name)))
-	 (spec (completing-read prompt-string ess-object-list)))
+	 (object-list (mapcar 'list (ess-get-object-list ess-local-process-name)))
+	 (spec (completing-read prompt-string object-list)))
     (list (cond
 	   ((string= spec "") default)
 	   (t spec)))))
