@@ -1,4 +1,4 @@
-;;; ess-site.el --- user customization of ess-mode
+;;; ess-site.el --- user customization of ESS
 
 ;; Copyright (C) 1993 David M. Smith
 ;; Copyright (C) 1997--2004 A.J. Rossini, Rich M. Heiberger, Martin
@@ -34,7 +34,7 @@
 ;;; directory location of this file must be supplied in
 ;;; ess-lisp-directory.	 The editing of remaining sections is
 ;;; optional.  It should then be byte-compiled, and users who wish to
-;;; use ess-mode should add the line:
+;;; use ESS should add the line:
 ;;;    (load "/PATH/TO/THIS/FILE/ess-site")
 ;;; (where /PATH/TO/THIS/FILE is the path to ess-site.elc: i.e. the
 ;;; value of ess-lisp-directory, below) to their .emacs file.
@@ -235,18 +235,6 @@ The ESS info directory stores the ESS info files.")
 
 (autoload 'Rd-mode "essddr" "Major mode for editing R documentation." t)
 
-;; remassoc exists as a built-in function in xemacs, but
-;; not in GNU emacs
-;;
-(if (not (functionp 'remassoc))
-    (defun remassoc (key a)
-      "remove an association pair from an alist"
-      (if a
-	  (let ((pair (car a)))
-	    (if (equal (car pair) key)
-		(cdr a)
-		(cons pair (remassoc key (cdr a))))))))
-
 ;; This is thanks to  Ed L Cashin <ecashin@uga.edu>, 03 Mar 2004 :
 (defun ess-restore-asm-extns ()
   "take away the S-Plus mode association for .s and .S files added by ESS
@@ -271,32 +259,31 @@ between .s or .S files and assembly mode.
 (if (assoc "\\.[rR]\\'" auto-mode-alist) nil
   (setq auto-mode-alist
 	(append
-	 '(("\\.sp\\'"	  . S-mode) ;; re: Don MacQueen <macq@llnl.gov>
-	   ("\\.[qsS]\\'" . S-mode) ;; q,s,S [see ess-restore-asm-extns above!]
-	   ("\\.ssc\\'"	  . S-mode) ;; Splus 4.x script files.
-	   ("\\.[rR]\\'"  . R-mode)
-	   ("\\.[rR]nw\\'"  . Rnw-mode)
+	 '(("\\.sp\\'"		. S-mode) ;; re: Don MacQueen <macq@llnl.gov>
+	   ("\\.[qsS]\\'"	. S-mode) ;; q,s,S [see ess-restore-asm-extns above!]
+	   ("\\.ssc\\'"		. S-mode) ;; Splus 4.x script files.
+	   ("\\.[rR]\\'"	. R-mode)
+	   ("\\.[rR]nw\\'"	. Rnw-mode)
 	   ("\\.[rR]profile\\'" . R-mode)
-	   ("NAMESPACE\\'" . R-mode)
-	   ("\\.omg\\'"		. omegahat-mode)
-	   ("\\.hat\\'"		. omegahat-mode) ;; Duncan's pref'd...
+	   ("NAMESPACE\\'"	. R-mode)
+	   ("\\.omg\\'"         . omegahat-mode)
+	   ("\\.hat\\'"         . omegahat-mode) ;; Duncan's pref'd...
 	   ("\\.lsp\\'"		. XLS-mode)
 	   ("\\.do\\'"		. STA-mode)
 	   ("\\.ado\\'"		. STA-mode)
-	   ("\\.sas\\'"		. SAS-mode)
-	   ("\\.SAS\\'"		. SAS-mode)
-	   ;;("\\.lst\\'"		. SAS-listing-mode);sasl
-	   ;; Too many *.log files, not only SAS :
-	   ;;("\\.log\\'"	. SAS-log-mode);sasl
+	   ("\\.[Ss][Aa][Ss]\\'"	. SAS-mode)
+	   ;; Many .log/.lst files, not just SAS
+	   ;;("\\.log\\'"	. SAS-log-mode)
+	   ;;("\\.lst\\'"	. SAS-listing-mode)
 	   ("\\.[Ss]t\\'"	. S-transcript-mode)
 	   ("\\.[Ss]out"	. S-transcript-mode)
 	   ("\\.[Rr]t\\'"	. R-transcript-mode)
 	   ("\\.[Rr]out"	. R-transcript-mode)
 	   ("\\.Rd\\'"		. Rd-mode)
-	   ("\\.[Bb][Uu][Gg]\\'"	 . ess-bugs-mode)
-	   ("\\.[Bb][Oo][Gg]\\'"	 . ess-bugs-mode)
-	   ("\\.[Bb][Mm][Dd]\\'"	 . ess-bugs-mode)
-	  )
+           ("\\.[Bb][Uu][Gg]\\'"         . ess-bugs-mode)
+           ("\\.[Bb][Oo][Gg]\\'"         . ess-bugs-mode)
+           ("\\.[Bb][Mm][Dd]\\'"         . ess-bugs-mode)
+          )
 	 auto-mode-alist)))
 
 ;; (1.4) Customize the dialects for your setup.
@@ -359,7 +346,7 @@ between .s or .S files and assembly mode.
 ;;; These commands are for running the PC version of Sqpe of S+4 and
 ;;; S+6 in an emacs buffer, using the same technology as ESS uses for
 ;;; Unix S-Plus.  Interactive graphics are unavailable in this mode.
-;;; See essd-sp4.el or essdsp6w.el
+;;; See essd-sp4.el or essd-sp6w.el
 
 ;;; These are the defaults.  Change them here if the defaults don't work.
 ;;; Use the 8.3 version of the pathname because embedded blanks will cause
@@ -385,7 +372,7 @@ between .s or .S files and assembly mode.
 ;;; control to the user.  On a 300 MHz machine with 96MB of RAM the
 ;;; delay is 60 seconds.  On a ???? MHz machine with 523MB the delay is
 ;;; 10 seconds.	 The user may need to adjust this number.
-(defvar ess-S+6-startup-delay 60
+(defvar ess-S+6-startup-delay 15 ;; <- 2005-01-03; MM
 "*Number of seconds to wait for the Commands window to appear before
 sending `inferior-ess-language-start' to S-Plus.")
 
@@ -411,20 +398,23 @@ sending `inferior-ess-language-start' to S-Plus.")
 
 (ess-message "[ess-site:] Before requiring dialect 'essd-** ....")
 (ess-message "[ess-site:] require 'essd-r ...")
-(require 'essd-r)    ;; S and common variants
-(ess-message "[ess-site:] require 'essd-s4 ...")
-(require 'essd-s4)
+(require 'essd-r)    ;; R
+;; (ess-message "[ess-site:] require 'essd-s4 ...")
+;; (require 'essd-s4) ; has become VERY RARE ..
+
 ;;(ess-message "[ess-site:] require 'essd-s3 ...")
 ;;(require 'essd-s3)  ; THIS IS RARE.  You probably do not have this.
+
+;; "sp" refers to S-PLUS (MathSoft/StatSci/Insightful):
 (ess-message "[ess-site:] require 'essd-sp3 ...")
-(require 'essd-sp3)  ;; "sp" refers to S-PLUS (MathSoft/StatSci).
+(require 'essd-sp3)
 
 (if ess-microsoft-p
     (progn
       (ess-message "[ess-site:] require 'essd-sp4 ...")
       (require 'essd-sp4)
-      (ess-message "[ess-site:] require 'essdsp6w ...")
-      (require 'essdsp6w))
+      (ess-message "[ess-site:] require 'essd-sp6w ...")
+      (require 'essd-sp6w))
   ;; else: decent OS
   (ess-message "[ess-site:] require 'essd-sp5 ...")
   (require 'essd-sp5)
@@ -456,7 +446,7 @@ sending `inferior-ess-language-start' to S-Plus.")
 (require 'ess-noweb)
 
 ;; ALWAYS:
-(require 'ess)
+(require 'ess); -> loads ess-cust.el and more
 
 (ess-write-to-dribble-buffer
    (format "[ess-site.el _2_]: ess-customize-alist=%s \n"
@@ -574,7 +564,8 @@ sending `inferior-ess-language-start' to S-Plus.")
 ;; The following two expressions automatically enable font-lock-mode
 ;; for ess-mode and inferior-ess-mode buffers.
 
-;; XEmacs has font-lock for ttys, as well.  So we need a better check!
+;; FIXME: XEmacs and Emacs 21.x has font-lock for ttys, as well.
+;; So we need a better check! [or do this unconditionally -working everywhere ??]
 (if window-system
     (progn
       (add-hook 'ess-mode-hook 'turn-on-font-lock t)
