@@ -1,15 +1,13 @@
 ;;; essd-sp4.el --- S-PLUS 4.x customization
-;;; Richard M. Heiberger, December 1998
 
-;; Copyright (C) 1998--1999 Richard M. Heiberger <rmh@fisher.stat.temple.edu>
+;; Copyright (C) 1998--2002 Richard M. Heiberger <rmh@fisher.stat.temple.edu>
+;; Copyright (C) 2003--2004 A.J. Rossini, Rich M. Heiberger, Martin
+;;	Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
-;; Author: Richard M. Heiberger <rmh@fisher.stat.temple.edu>
-;; Maintainer: A.J. Rossini <rossini@biostat.washington.edu>
+;; Original Author: Richard M. Heiberger <rmh@fisher.stat.temple.edu>
 ;; Created: December 1998
-;; Modified: $Date: 2002/06/24 07:48:01 $
-;; Version: $Revision: 1.14 $
-;; RCS: $Id: essd-sp4.el,v 1.14 2002/06/24 07:48:01 maechler Exp $
-;;
+;; Maintainers: ESS-core <ESS-core@stat.math.ethz.ch>
+
 ;; Keywords: start up, configuration.
 
 ;; This file is part of ESS.
@@ -21,11 +19,11 @@
 
 ;; This file is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
+;; along with GNU Emacs; see the file COPYING.	If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 ;;; Commentary:
@@ -38,7 +36,7 @@
 (require 'ess-iw32)
 
 (autoload 'inferior-ess "ess-inf" "Run an ESS process.")
-(autoload 'ess-mode     "ess-mode" "Edit an ESS process.")
+(autoload 'ess-mode	"ess-mode" "Edit an ESS process.")
 
 ;;; Code:
 
@@ -52,116 +50,79 @@ alternative nil uses an existing S+4 GUI (if there is one) and
 connects it to the '(ddeESS [S+4])' window.")
 
 (defvar S+4-customize-alist
-  '((ess-local-customize-alist     . 'S+4-customize-alist)
-    (ess-language                  . "S")
-    (ess-dialect                   . S+4-dialect-name)
-    (ess-suffix                    . "S")
-    (ess-dump-filename-template    . (concat (user-login-name)
-					     ".%s."
-					     ess-suffix))
-    (ess-mode-editing-alist        . S-editing-alist)
-    (ess-mode-syntax-table         . S-syntax-table)
-    (ess-help-sec-regex            . ess-help-S+-sec-regex)
-    (ess-help-sec-keys-alist       . S+-help-sec-keys-alist)
-    (ess-loop-timeout              . 500000 )
-    (ess-object-name-db-file       . "ess-sp4-namedb.el" )
-    (ess-retr-lastvalue-command
-     . ".Last.value <- get(\".ess.lvsave\",frame=0)\n")
-    (ess-save-lastvalue-command
-     . "assign(\".ess.lvsave\",.Last.value,frame=0)\n")
-    (inferior-ess-program          . inferior-S+4-program-name)
-;;    (inferior-ess-ddeclient        . "ddeclient")
-;;    (inferior-ess-client-name      . "S-PLUS")
-;;    (inferior-ess-client-command   . "SCommand")
-    (inferior-ess-objects-command  . "objects(%d)\n")
-    (inferior-ess-help-command     . "help(\"%s\")\n")
-    (inferior-ess-exit-command     . "q()\n")
-    (inferior-ess-primary-prompt   . "[a-zA-Z0-9() ]*> ?")
-    (inferior-ess-secondary-prompt . "+ ?")
-    (comint-use-prompt-regexp-instead-of-fields . t) ;; emacs 21 and up
-    (inferior-ess-start-file       . nil) ;"~/.ess-S+4")
-    (inferior-ess-start-args       . (concat
-				      inferior-S+4-multipleinstances
-				      " "
-				      inferior-S+4-print-command
-				      " S_PROJ="
-				      (directory-file-name default-directory))
-				   )
-    (ess-STERM  . "ddeESS")
-    (ess-editor . S-editor)
-    (ess-pager  . S-pager)
-    (inferior-ess-language-start . (eval inferior-S-language-start))
-    )
- "Variables to customize for S+4")
+  (append
+   '((ess-local-customize-alist		. 'S+4-customize-alist)
+     (ess-dialect			. S+4-dialect-name)
+     (ess-loop-timeout			. ess-S-loop-timeout);fixme: dialect spec.
+     (ess-object-name-db-file		. "ess-sp4-namedb.el" )
+     (inferior-ess-program		. inferior-S+4-program-name)
+     (inferior-ess-help-command		. "help(\"%s\")\n")
+
+     (inferior-ess-start-file		. nil) ;"~/.ess-S+4")
+     (inferior-ess-start-args		. (concat
+					   inferior-S+4-multipleinstances " "
+					   inferior-S+4-print-command
+					   " S_PROJ="
+					   (directory-file-name default-directory)))
+     ;;	   (inferior-ess-ddeclient	. "ddeclient")
+     ;;	   (inferior-ess-client-name	. "S-PLUS")
+     ;;	   (inferior-ess-client-command	. "SCommand")
+     (ess-STERM	 . "ddeESS")
+     )
+   S+common-cust-alist)
+
+  "Variables to customize for S+4")
 
 (defvar Sqpe+4-customize-alist
-  '((ess-local-customize-alist     . 'Sqpe+4-customize-alist)
-    (ess-language                  . "S")
-    (ess-dialect                   . S+4-dialect-name)
-    (ess-suffix                    . "S")
-    (ess-dump-filename-template    . (concat (user-login-name)
-					     ".%s."
-					     ess-suffix))
-    (ess-mode-editing-alist        . S-editing-alist)
-    (ess-mode-syntax-table         . S-syntax-table)
-    (ess-help-sec-regex            . ess-help-S+-sec-regex)
-    (ess-help-sec-keys-alist       . S+-help-sec-keys-alist)
-    (ess-loop-timeout              . 500000 )
-    (ess-object-name-db-file       . "ess-sp4-namedb.el" )
-    (ess-retr-lastvalue-command
-     . ".Last.value <- get(\".ess.lvsave\",frame=0)\n")
-    (ess-save-lastvalue-command
-     . "assign(\".ess.lvsave\",.Last.value,frame=0)\n")
-    (inferior-ess-program          . inferior-Sqpe+4-program-name)
-    (inferior-ess-objects-command  . "objects(%d)\n")
-    (inferior-ess-help-command     . "help(\"%s\")\n")
-    (inferior-ess-exit-command     . "q()\n")
-    (inferior-ess-primary-prompt   . "[a-zA-Z0-9() ]*> ?")
-    (inferior-ess-secondary-prompt . "+ ?")
-    (comint-use-prompt-regexp-instead-of-fields . t) ;; emacs 21 and up
-    (inferior-ess-start-file       . nil) ;"~/.ess-S+4")
-    (inferior-ess-start-args       . "")
-    (ess-STERM  . "iESS")
-    (ess-editor . S-editor)
-    (ess-pager  . S-pager)
-    (inferior-ess-language-start .
-				 (concat "options("
-					 "STERM='"  ess-STERM  "'"
-					 (if ess-editor
-					     (concat ", editor='" ess-editor "'"))
-					 (if ess-pager
-					     (concat ", pager='"  ess-pager  "'"))
-					 ")"))
-)
- "Variables to customize for Sqpe+4.")
+  (append
+   '((ess-local-customize-alist		. 'Sqpe+4-customize-alist)
+     (ess-dialect			. S+4-dialect-name)
+     (ess-loop-timeout			. 500000 );fixme: dialect specific custom.v
+     (ess-object-name-db-file		. "ess-sp4-namedb.el" )
+     (inferior-ess-program		. inferior-Sqpe+4-program-name)
+     (inferior-ess-help-command		. "help(\"%s\")\n")
+     (inferior-ess-search-list-command	. "searchPaths()\n")
+     (inferior-ess-start-file		. nil) ;"~/.ess-S+4")
+     (inferior-ess-language-start . (concat
+				     "options("
+				     "STERM='" ess-STERM "'"
+				     (if ess-editor
+					 (concat ", editor='" ess-editor "'"))
+				     (if ess-pager
+					 (concat ", pager='"  ess-pager	 "'"))
+				     ")"))
+     (ess-STERM	 . "iESS")
+     )
+   S+common-cust-alist)
 
+  "Variables to customize for Sqpe+4.")
 
 
 ;;; There are extra complications in S+4 (compared to S+3) because
 ;;;
 ;;; (1) The StatSci supplied Splus.exe doesn't work in an emacs
-;;;     buffer.  It works as as a GUI window and we must send commands
-;;;     to it through ddeclient.  Nonetheless, we need to give it a
-;;;     process name and be sure that that there is a valid running
-;;;     process in the '(ddeESS [S+4])' buffer.  Therefore we create an
-;;;     ESS process in the buffer as a placeholder and start a shell
-;;;     in the ESS buffer.  From the shell we start Splus.  Once Splus
-;;;     finishes initializing and kills the original shell, we start
-;;;     another shell.  We have a buffer-local variable
-;;;     inferior-ess-ddeclient, initialized to nil.  When there is a
-;;;     non-nil value of inferior-ess-ddeclient we send lines to
-;;;     inferior-ess-ddeclient rather than to the Splus process.
+;;;	buffer.	 It works as as a GUI window and we must send commands
+;;;	to it through ddeclient.  Nonetheless, we need to give it a
+;;;	process name and be sure that that there is a valid running
+;;;	process in the '(ddeESS [S+4])' buffer.	 Therefore we create an
+;;;	ESS process in the buffer as a placeholder and start a shell
+;;;	in the ESS buffer.  From the shell we start Splus.  Once Splus
+;;;	finishes initializing and kills the original shell, we start
+;;;	another shell.	We have a buffer-local variable
+;;;	inferior-ess-ddeclient, initialized to nil.  When there is a
+;;;	non-nil value of inferior-ess-ddeclient we send lines to
+;;;	inferior-ess-ddeclient rather than to the Splus process.
 ;;; (2) There is no Splus process running in the '(ddeESS [S+4])'
-;;;     buffer.  Therefore inferior-ess will never see a prompt,
-;;;     unless we first change it to the null prompt "^".  Then once
-;;;     the process has started, we change it back.
+;;;	buffer.	 Therefore inferior-ess will never see a prompt,
+;;;	unless we first change it to the null prompt "^".  Then once
+;;;	the process has started, we change it back.
 ;;; (3) When M-x S+4 starts Splus by a shell command, then Splus is an
-;;;     independent process and will be survive if the '(ddeESS [S+4])'
-;;;     buffer is killed (or emacs is quit).  The '(ddeESS [S+4])' is
-;;;     made read-only and a warning is placed in it saying that "You
-;;;     can't type anything here."  Actually, if the standalone Splus
-;;;     is killed and the '(ddeESS [S+4])' is made writable (C-x C-q),
-;;;     then '(ddeESS [S+4])' becomes a shell buffer.
+;;;	independent process and will be survive if the '(ddeESS [S+4])'
+;;;	buffer is killed (or emacs is quit).  The '(ddeESS [S+4])' is
+;;;	made read-only and a warning is placed in it saying that "You
+;;;	can't type anything here."  Actually, if the standalone Splus
+;;;	is killed and the '(ddeESS [S+4])' is made writable (C-x C-q),
+;;;	then '(ddeESS [S+4])' becomes a shell buffer.
 ;;;
 (defun S+4 (&optional proc-name)
   "Call 'S-PLUS 4.x', the 'GUI Thing' from StatSci.  Put S-Plus in an
@@ -169,7 +130,7 @@ independent MS-Window (Splus persists even if the '(ddeESS [S+4])'
 window is killed in emacs).  Do this by creating a comint process that
 calls sh.  Send a shell command in that sh buffer to call Splus.  When
 it completes set up a shell as a placeholder in the '(ddeESS [S+4])'
-buffer.  The S-Plus options are correctly set.  In particular, the
+buffer.	 The S-Plus options are correctly set.	In particular, the
 S-Plus Commands window is opened if the Options/General
 Settings/Startup menu says it should be.  There is a 30 second delay
 during startup in which the screen will not be refreshed.  This delay
@@ -181,7 +142,7 @@ is here to allow slow disks to start the Splus program."
      (format "\n(S+4): ess-dialect=%s, buf=%s\n" ess-dialect
 	     (current-buffer)))
     (setq ess-customize-alist		; change inferior-ess-program
-	  (append ess-customize-alist '((inferior-ess-program   . "sh"))))
+	  (append ess-customize-alist '((inferior-ess-program	. "sh"))))
     (setq ess-customize-alist		; change inferior-ess-primary-prompt
 	  (append ess-customize-alist '((inferior-ess-primary-prompt   . "^"))))
     (setq ess-customize-alist		; change inferior-ess-start-args
@@ -203,21 +164,21 @@ is here to allow slow disks to start the Splus program."
     (ess-setq-vars-local ess-customize-alist)
 ;;; the next three lines belong in customize-alist, but can't be there
 ;;; because of the broken ess-setq-vars-default usage in ess-inf.el
-    (setq inferior-ess-ddeclient         "ddeclient")
-    (setq inferior-ess-client-name       "S-PLUS")
-    (setq inferior-ess-client-command    "SCommand")
+    (setq inferior-ess-ddeclient	 "ddeclient")
+    (setq inferior-ess-client-name	 "S-PLUS")
+    (setq inferior-ess-client-command	 "SCommand")
 ;;; end of what belongs in customize-alist
     (setq comint-process-echoes nil)
     (setq comint-input-sender 'comint-simple-send)
     (goto-char (point-max))
     (insert (concat inferior-S+4-program-name " "
 		    inferior-ess-start-args)) ; Note: there is no final "&".
-    ;; Without the "&", the results of  !system.command  come to '(ddeESS [S+4])'
+    ;; Without the "&", the results of	!system.command	 come to '(ddeESS [S+4])'
     ;; With the "&", the results of  !system.command  in S get lost.
     (inferior-ess-send-input)
     (sleep-for 30) ; Need to wait, else working too fast!
-                   ; If the ess-current-process-name doesn't appear in the
-       		   ; Splus Commands window increase the sleep-for time!
+		   ; If the ess-current-process-name doesn't appear in the
+		   ; Splus Commands window increase the sleep-for time!
     (setq ess-local-process-name ess-current-process-name)
     (ess-eval-linewise (concat "#" ess-current-process-name))
     (beginning-of-buffer)
@@ -232,7 +193,7 @@ Splus/Window/Commands Window).\n
 Any results of the   !system.command   typed at the S prompt in the
 Splus Commands window appear in this buffer.\n\n")
     (goto-char (point-max))		; comint-mode-map makes '(ddeESS [S+4])'
-;;  (use-local-map comint-mode-map)     ;a shell buffer after Splus is finished.
+;;  (use-local-map comint-mode-map)	;a shell buffer after Splus is finished.
     (set-buffer-process-coding-system 'raw-text-dos 'raw-text-unix)
     (toggle-read-only t)		; force buffer to be read-only
     (setq mode-name "ddeESS")
@@ -259,14 +220,14 @@ If you have a HOME environment variable, it will open it there."
     ;; It doesn't know about us, so nothing comes back.
     (S+4 proc-name))
   (save-excursion
-    (set-buffer (car (buffer-list)))    ; get the ESS buffer just created
+    (set-buffer (car (buffer-list)))	; get the ESS buffer just created
     (toggle-read-only nil)		; permit writing in ESS buffer
     (goto-char (point-max))
     (beginning-of-line)
     (forward-line -1)
     (insert
      "This is S+4-existing.
-Results of the   !system.command   typed at the S prompt in the
+Results of the	 !system.command   typed at the S prompt in the
 Splus Commands window blink a DOS window and you won't see them.\n\n")
     (toggle-read-only t)		; restore ESS buffer to be read-only
     ))
@@ -274,11 +235,11 @@ Splus Commands window blink a DOS window and you won't see them.\n\n")
 
 ;;; There are extra complications in Sqpe+4 (compared to S+3) because
 ;;; (1) The StatSci supplied Sqpe.exe won't work without SHOME as an
-;;;     environment variable and Sqpe does not take command line
-;;;     arguments and
+;;;	environment variable and Sqpe does not take command line
+;;;	arguments and
 ;;; (2) Sqpe.exe comes up with options(interactive=F), which means it
-;;;     doesn't provide prompts by default, and we must change it to T so
-;;;     it will provide prompts.
+;;;	doesn't provide prompts by default, and we must change it to T so
+;;;	it will provide prompts.
 ;;;
 (defun Sqpe+4 (&optional proc-name)
   "Call 'Sqpe' from 'S-PLUS 4.x', the 'Real Thing'  from StatSci."
@@ -293,16 +254,16 @@ Splus Commands window blink a DOS window and you won't see them.\n\n")
 	  (append ess-customize-alist '((inferior-ess-primary-prompt   . "^"))))
     (inferior-ess)
     (setq ess-customize-alist Sqpe+4-customize-alist) ; restore i-e-p-p in alist
-    (ess-setq-vars-local ess-customize-alist)    ; restore i-e-p-p in buffer
-    (setq inferior-ess-prompt                    ; define with correct i-e-p-p
-	  ;; Do not anchor to bol with `^'       ; (copied from ess-inf.el)
+    (ess-setq-vars-local ess-customize-alist)	 ; restore i-e-p-p in buffer
+    (setq inferior-ess-prompt			 ; define with correct i-e-p-p
+	  ;; Do not anchor to bol with `^'	 ; (copied from ess-inf.el)
 	  (concat "\\("
 		  inferior-ess-primary-prompt
 		  "\\|"
 		  inferior-ess-secondary-prompt
 		  "\\)"))
     (setq comint-prompt-regexp (concat "^" inferior-ess-prompt))
-                                                ; define with correct i-e-p-p
+						; define with correct i-e-p-p
     (setq comint-input-sender 'inferior-ess-input-sender)
     (add-hook 'comint-output-filter-functions 'shell-strip-ctrl-m nil t)
     (goto-char (point-max))
@@ -321,7 +282,7 @@ Splus Commands window blink a DOS window and you won't see them.\n\n")
   (interactive)
   (setq ess-customize-alist S+4-customize-alist)
   (ess-mode S+4-customize-alist proc-name)
-  (if ess-S-use-imenu (ess-imenu-R)))
+  (if ess-imenu-use-S (ess-imenu-R)))
 
 (defun S+4-transcript-mode ()
   "S-PLUS 4.x transcript mode."
@@ -335,7 +296,7 @@ independent MS-Window (Splus persists even if the '(ddeESS [S+4])'
 window is killed in emacs).  Do this by creating a comint process that
 calls sh.  Send a shell command in that sh buffer to call Splus.  When
 it completes set up a shell as a placeholder in the '(ddeESS [S+4])'
-buffer.  The S-Plus options are correctly set.  In particular, the
+buffer.	 The S-Plus options are correctly set.	In particular, the
 S-Plus Commands window is opened if the Options/General
 Settings/Startup menu says it should be.  There is a 30 second delay
 during startup in which the screen will not be refreshed.  This delay
@@ -365,9 +326,9 @@ is here to allow slow disks to start the Splus program."
     (ess-setq-vars-local ess-customize-alist)
 ;;; the next three lines belong in customize-alist, but can't be there
 ;;; because of the broken ess-setq-vars-default usage in ess-inf.el
-    (setq inferior-ess-ddeclient         "ddeclient")
-    (setq inferior-ess-client-name       "S-PLUS")
-    (setq inferior-ess-client-command    "SCommand")
+    (setq inferior-ess-ddeclient	 "ddeclient")
+    (setq inferior-ess-client-name	 "S-PLUS")
+    (setq inferior-ess-client-command	 "SCommand")
 ;;; end of what belongs in customize-alist
     (setq comint-input-sender 'comint-simple-send)
     (setq comint-process-echoes nil)
@@ -376,11 +337,11 @@ is here to allow slow disks to start the Splus program."
     (insert (concat inferior-S+4-program-name " "
 		    inferior-ess-start-args)) ; Note: there is no final "&".
 ; Without the "&", the results of  !system.command  come to '(ddeESS [S+4])'
-; With the "&", the results of  !system.command  in S get lost.
+; With the "&", the results of	!system.command	 in S get lost.
     (inferior-ess-send-input)
     (sleep-for 30) ; Need to wait, else working too fast!
-                   ; If the ess-current-process-name doesn't appear in the
-       		   ; Splus Commands window increase the sleep-for time!
+		   ; If the ess-current-process-name doesn't appear in the
+		   ; Splus Commands window increase the sleep-for time!
 ;;; from msdos-minor-mode
   (setq comint-process-echoes t)
   (add-hook 'comint-output-filter-functions 'shell-strip-ctrl-m nil t)
@@ -425,14 +386,14 @@ If you have a HOME environment variable, it will open it there."
   (let* ((inferior-S+4-multipleinstances ""))
     (S+4-msdos proc-name))
   (save-excursion
-    (set-buffer (car (buffer-list)))    ; get the ESS buffer just created
+    (set-buffer (car (buffer-list)))	; get the ESS buffer just created
     (toggle-read-only nil)		; permit writing in ESS buffer
     (goto-char (point-max))
     (beginning-of-line)
     (forward-line -1)
     (insert
      "This is S+4-msdos-existing.
-Results of the   !system.command   typed at the S prompt in the
+Results of the	 !system.command   typed at the S prompt in the
 Splus Commands window blink a DOS window and you won't see them.\n\n")
     (toggle-read-only t)		; restore ESS buffer to be read-only
     ))
@@ -445,11 +406,11 @@ Splus Commands window blink a DOS window and you won't see them.\n\n")
 
 ;;; This file is automatically placed in Outline minor mode.
 ;;; The file is structured as follows:
-;;; Chapters:     ^L ;
-;;; Sections:    ;;*;;
+;;; Chapters:	  ^L ;
+;;; Sections:	 ;;*;;
 ;;; Subsections: ;;;*;;;
-;;; Components:  defuns, defvars, defconsts
-;;;              Random code beginning with a ;;;;* comment
+;;; Components:	 defuns, defvars, defconsts
+;;;		 Random code beginning with a ;;;;* comment
 
 ;;; Local variables:
 ;;; mode: emacs-lisp
