@@ -1,14 +1,12 @@
 ;;; essd-sp6.el --- S-Plus 6  customization
 
-;; Copyright (C) 2001 A.J. Rossini <rossini@u.washington.edu>
+;; Copyright (C) 2001--2004 A.J. Rossini, Rich M. Heiberger, Martin
+;;	Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
-;; Author: A.J. Rossini <rossini@u.washington.edu>
-;; Maintainer: ESS Core Team <ESS-core@stat.math.ethz.ch>
+;; Original Author: A.J. Rossini <rossini@u.washington.edu>
 ;; Created: 2001/02/06
-;; Modified: $Date: 2002/06/24 07:48:01 $
-;; Version: $Revision: 1.10 $
-;; RCS: $Id: essd-sp6.el,v 1.10 2002/06/24 07:48:01 maechler Exp $
-;;
+;; Maintainer: ESS Core Team <ESS-core@stat.math.ethz.ch>
+
 ;; Keywords: start up, configuration.
 
 ;; This file is part of ESS.
@@ -75,48 +73,24 @@
             ))))
 
 (defvar S+6-customize-alist
-  '((ess-local-customize-alist     . 'S+6-customize-alist)
-    (ess-language                  . "S")
-    (ess-dialect                   . S+6-dialect-name)
-    (ess-suffix                    . "S")
-    (ess-directory-function        . S+6-directory-function)
-    (ess-setup-directory-function  . S+6-setup-directory-function)
-    (ess-dump-filename-template    . (concat (user-login-name)
-					     ".%s."
-					     ess-suffix))
-    (ess-mode-editing-alist        . S-editing-alist)
-    (ess-mode-syntax-table         . S-syntax-table)
-    (ess-help-sec-regex            . ess-help-S+-sec-regex)
-					;or just "^[A-Z. ---]+:$"
-    (ess-help-sec-keys-alist       . S+-help-sec-keys-alist)
+  (append
+   '((ess-local-customize-alist		. 'S+6-customize-alist)
+     (ess-dialect			. S+6-dialect-name)
+     (ess-loop-timeout			. ess-S-loop-timeout);fixme: dialect spec.
+     (ess-function-pattern              . ess-R-function-pattern)
+
+     (ess-object-name-db-file		. "ess-sp6-namedb.el")
+     (inferior-ess-program		. inferior-S+6-program-name)
+     (inferior-ess-help-command	  . "help(\"%s\",pager=\"slynx -dump\",window=F)\n")
+     (inferior-ess-search-list-command	. "searchPaths()\n")
+
+     (ess-directory-function		. S+6-directory-function)
+     (ess-setup-directory-function	. S+6-setup-directory-function)
+     (inferior-ess-start-args       . inferior-Splus-args)
+     (ess-STERM	 . "iESS")
+     )
+   S+common-cust-alist)
 
-    (ess-function-template         . " <- \n#\nfunction()\n{\n\n}\n")
-    (ess-loop-timeout              . 500000 )
-    (ess-object-name-db-file       . "ess-sp6-namedb.el")
-    (ess-dumped-missing-re
-     . "\\(\\(<-\\|=\\)\nDumped\n\\'\\)\\|\\(\\(<-\\|=\\)\\(\\s \\|\n\\)*\\'\\)")
-    (ess-syntax-error-re
-     . "\\(Syntax error: .*\\) at line \\([0-9]*\\), file \\(.*\\)$")
-    (ess-retr-lastvalue-command
-     . ".Last.value <- get(\".ess.lvsave\",frame=0)\n")
-    (ess-save-lastvalue-command
-     . "assign(\".ess.lvsave\",.Last.value,frame=0)\n")
-    (inferior-ess-program          . inferior-S+6-program-name)
-    (inferior-ess-objects-command  . "objects(where = %d)\n")
-    (inferior-ess-objects-pattern  . ".*") ; for new s4 stuff
-    (inferior-ess-help-command     . "help(\"%s\",pager=\"slynx -dump\",window=F)\n")
-    ;; "paths": get the "/" needed by  (ess-dir-modtime dir)  in ./ess-inf.el:
-    ;; (inferior-ess-search-list-command . "search(\"paths\")\n")
-    (inferior-ess-search-list-command . "search()\n")
-    (inferior-ess-exit-command     . "q()\n")
-    (comint-use-prompt-regexp-instead-of-fields . t) ;; emacs 21 and up
-    (inferior-ess-primary-prompt   . "[a-zA-Z0-9() ]*> ?")
-    (inferior-ess-secondary-prompt . "+ ?")
-    (ess-STERM  . "iESS")
-    (ess-editor . S-editor)
-    (ess-pager  . S-pager)
-    (inferior-ess-language-start . (eval inferior-S-language-start))
-    )
   "Variables to customize for S+6.")
 
 
@@ -137,7 +111,8 @@ New way to do it."
   (interactive)
   (setq ess-customize-alist S+6-customize-alist)
   (ess-mode S+6-customize-alist proc-name)
-  (if ess-S-use-imenu (ess-imenu-S)))
+  (if (fboundp 'ess-add-toolbar) (ess-add-toolbar))
+  (if ess-imenu-use-S (ess-imenu-S)))
 
 (defun S+6-transcript-mode ()
   "S-PLUS 6 transcript mode."
