@@ -2,7 +2,7 @@
 
 ;; Copyright (C) 1989-1997 D. Bates, Kademan, Ritter, D.M. Smith, K. Hornik,
 ;;	R.M. Heiberger, M. Maechler, and A.J. Rossini.
-;; Copyright (C) 1998-2004 A.J. Rossini, Rich M. Heiberger, Martin
+;; Copyright (C) 1998-2005 A.J. Rossini, Rich M. Heiberger, Martin
 ;;	Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
 ;; Original Author: A.J. Rossini <rossini@biostat.washington.edu>
@@ -57,19 +57,20 @@
   ;;(modify-syntax-entry ?.  "w"  S-syntax-table) ; "." used in S obj names
   (modify-syntax-entry ?.  "_"	S-syntax-table) ; see above/below,
 					; plus consider separation.
-  (modify-syntax-entry ?$  "_"	S-syntax-table) ; foo.bar$hack is 1 symbol
+  (modify-syntax-entry ?$  "_"	S-syntax-table); foo$comp = 1 symbol(completion)
+  (modify-syntax-entry ?@  "_"	S-syntax-table); foo@slot = 1 symbol(completion)
   (modify-syntax-entry ?_  "."	S-syntax-table)
   (modify-syntax-entry ?*  "."	S-syntax-table)
   (modify-syntax-entry ?<  "."	S-syntax-table)
   (modify-syntax-entry ?>  "."	S-syntax-table)
   (modify-syntax-entry ?/  "."	S-syntax-table))
 
-(defvar S-editing-alist
+(defvar R-editing-alist
   '((paragraph-start		  . (concat "\\s-*$\\|" page-delimiter))
     (paragraph-separate		  . (concat "\\s-*$\\|" page-delimiter))
     (paragraph-ignore-fill-prefix . t)
     (require-final-newline	  . t)
-    (comment-start		  . "#")
+    (comment-start		  . "##")
     (comment-start-skip		  . "#+ *")
     (comment-column		  . 40)
     ;;(comment-indent-function	. 'S-comment-indent)
@@ -84,10 +85,19 @@
     (ess-mode-syntax-table	  . S-syntax-table)
     ;; For Changelog add, require ' ' before <- : "attr<-" is a function name :
     (add-log-current-defun-header-regexp . "^\\(.+\\)\\s-+<-[ \t\n]*function")
-    (font-lock-defaults		  . '(ess-mode-font-lock-keywords
-				      nil nil ((?\. . "w"))))
+    (font-lock-defaults		  . '(ess-R-mode-font-lock-keywords
+				      nil nil ((?\. . "w") (?\_ . "w"))))
     )
-  "General options for editing S, S+, and R source files.")
+  "General options for R source files.")
+
+(defvar S-editing-alist
+  ;; copy the R-list and modify :
+  (let ((S-alist (copy-alist R-editing-alist)))
+    (setcdr (assoc 'font-lock-defaults S-alist)
+	    (quote '(ess-S-mode-font-lock-keywords nil nil ((?\. . "w")))))
+    ;;      ^^ extra quote is needed - why?
+    S-alist)
+  "General options for editing S and S+ source files.")
 
 (defvar inferior-S-language-start
   '(concat "options("
@@ -97,9 +107,58 @@
 	     ")")
   "S language expression for startup -- default for all S dialects.")
 
+(defconst S-common-cust-alist
+  '((ess-language                  . "S")
+    (inferior-ess-exit-command     . "q()\n")
+    (inferior-ess-language-start   . (eval inferior-S-language-start))
+    ;;harmful for shell-mode's C-a: -- but "necessary" for ESS-help(?) :
+    (comint-use-prompt-regexp-instead-of-fields . t) ;; emacs 21 and up
+  )
+  "S-language common settings for all <dialect>-customize-alist s"
+)
+
+(defconst S+common-cust-alist
+  (append
+   '((ess-suffix                . "S")
+     (ess-mode-syntax-table     . S-syntax-table)
+     (ess-help-sec-regex	. ess-help-S+-sec-regex)
+     (ess-help-sec-keys-alist	. ess-help-S+sec-keys-alist)
+     (ess-change-sp-regexp	. ess-S+-change-sp-regexp)
+     (ess-cmd-delay		. (if (featurep 'xemacs); needs much less delay
+				      (* 0.1 ess-S+-cmd-delay)
+				    ess-S+-cmd-delay))
+     (ess-function-pattern      . ess-S-function-pattern)
+     (ess-function-template	. " <- \n#\nfunction()\n{\n\n}\n")
+     (ess-dump-filename-template . (ess-replace-regexp-in-string
+				    "S$" ess-suffix ; in the one from custom:
+				    ess-dump-filename-template-proto))
+
+     (ess-mode-editing-alist	. S-editing-alist)
+
+     (ess-dumped-missing-re
+      . "\\(\\(<-\\|=\\)\nDumped\n\\'\\)\\|\\(\\(<-\\|=\\)\\(\\s \\|\n\\)*\\'\\)")
+     (ess-syntax-error-re
+      . "\\(Syntax error: .*\\) at line \\([0-9]*\\), file \\(.*\\)$")
+     (ess-retr-lastvalue-command
+      . ".Last.value <- get(\".ess.lvsave\",frame=0)\n")
+     (ess-save-lastvalue-command
+      . "assign(\".ess.lvsave\",.Last.value,frame=0)\n")
+
+     (inferior-ess-objects-command  . inferior-Splus-objects-command)
+     (inferior-ess-font-lock-keywords . inferior-ess-S-font-lock-keywords)
+     (inferior-ess-primary-prompt   . "[a-zA-Z0-9() ]*> ?")
+     (inferior-ess-secondary-prompt . "+ ?")
+
+     (ess-editor . S-editor)
+     (ess-pager  . S-pager)
+     )
+   S-common-cust-alist)
+  "Common settings for all S+<*>-customize-alist s"
+)
+
 ;;; Changes from S to S-PLUS 3.x.  (standard S3 should be in essl-s!).
 
-(defconst S+-help-sec-keys-alist
+(defconst ess-help-S+sec-keys-alist
   '((?a . "ARGUMENTS:")
     (?b . "BACKGROUND:")
     (?B . "BUGS:")
@@ -120,7 +179,7 @@
 ;;; regexp-search, and so specials should be quoted.
 
 ;; S ver.3 (NOT S-Plus)
-(defconst S3-help-sec-keys-alist
+(defconst ess-help-S3-sec-keys-alist
   '((?a . "ARGUMENTS:")
     (?b . "BACKGROUND:")
     (?B . "BUGS:")
@@ -136,7 +195,7 @@
   "Help section keys for S ver.3.")
 
 ;; S ver.4 (NOT S-Plus)
-(defconst S4-help-sec-keys-alist
+(defconst ess-help-S4-sec-keys-alist
   '((?a . "ARGUMENTS:")
     (?b . "BACKGROUND:")
     (?B . "BUGS:")
@@ -152,7 +211,7 @@
   "Help section keys for S4.")
 
 ;; R
-(defconst R-help-sec-keys-alist
+(defconst ess-help-R-sec-keys-alist
   '((?a . "\\s *Arguments:")
     (?d . "\\s *Description:")
     (?D . "\\s *Details:")
@@ -181,7 +240,7 @@
 when \\<ess-mode-map>\\[ess-insert-function-outline] is used.
 Placeholders (substituted `at runtime'): $A$ for `Author', $D$ for `Date'.")
 
-;; Use the user's own ~/S/emacs-fun.outline  is (s)he has one : ---
+;; Use the user's own ~/S/emacs-fun.outline  if (s)he has one : ---
 (let ((outline-file (concat (getenv "HOME") "/S/function-outline.S")))
   (if (file-exists-p outline-file)
       (setq ess-function-outline-file outline-file)))
@@ -522,9 +581,9 @@ operator is removed and replaced by the underscore.  `ess-S-assign',
 typically \" <- \", can be customized.	In ESS modes other than R/S,
 an underscore is always inserted. "
   (interactive)
-  ;;(insert (if (inside-string/comment-p (point)) "_" ess-S-assign))
+  ;;(insert (if (ess-inside-string-or-comment-p (point)) "_" ess-S-assign))
   (if (or
-       (inside-string/comment-p (point))
+       (ess-inside-string-or-comment-p (point))
        (not (equal ess-language "S")))
       (insert "_")
     ;; Else one keypress produces ess-S-assign; a second keypress will delete
@@ -595,6 +654,12 @@ and I need to relearn emacs lisp (but I had to, anyway."
     (set-syntax-table ess-mode-syntax-table)
     ))
 
+(add-hook 'ess-mode-hook
+	  (lambda ()
+	    (set (make-local-variable 'fill-nobreak-predicate)
+		 'ess-inside-string-p)
+	    (set (make-local-variable 'auto-fill-function)
+		 'ess-do-auto-fill)))
 
 (provide 'essl-s)
 
