@@ -105,15 +105,10 @@ the mode line."
   "Fundamental mode with `ess-listing-minor-mode' and read-only."
   (interactive)
   (fundamental-mode)
-  (ess-listing-minor-mode t)
+  (ess-listing-minor-mode 1)
   (use-local-map sas-mode-local-map)
   (toggle-read-only t)) ;; to protect the buffer.
 
-(if (not (featurep 'xemacs))
-    (setq auto-mode-alist
-	(append '(("\\.[lL][sS][tT]\\'" . sas-listing-mode)) auto-mode-alist)))
-;;;###autoload(add-to-list 'auto-mode-alist '("\\.[lL][sS][tT]\\'" . sas-listing-mode))
-
 (fset 'sas-log-mode        'SAS-log-mode)
 (fset 'SAS-transcript-mode 'SAS-log-mode)
 (fset 'sas-transcript-mode 'SAS-log-mode)
@@ -432,7 +427,7 @@ number."
 		"\\|i\\(d\\|f\\|n\\(dex\\|f\\(ile\\|ormat\\)\\|put\\|value\\)\\)"
 		"\\|keep\\|l\\(abel\\|ength\\|i\\(bname\\|nk\\|st\\)\\|smeans\\)"
 		"\\|m\\(anova\\|e\\(ans\\|rge\\)\\|issing\\|od\\(el\\|ify\\)\\)\\|note"
-		"\\|o\\(ptions\\|therwise\\|utput\\)\\|p\\(lot\\|ut\\)"
+		"\\|o\\(ptions\\|therwise\\|utput\\)\\|p\\(arms\\|lot\\|ut\\)"
 		"\\|r\\(andom\\|e\\(name\\|peated\\|tain\\)\\)"
 		"\\|s\\(ave\\|e\\(lect\\|t\\)\\|kip\\|trata\\|umby\\)"
 		"\\|t\\(ables?\\|i\\(me\\|tle\\(10?\\|[2-9]\\)?\\)\\)\\|update"
@@ -464,7 +459,9 @@ number."
 		"\\([1-9][0-9]?\\)?\\>")	    font-lock-keyword-face)
 
 	 ;; SAS Datastep functions and SAS macro functions
-	 (cons "%[a-z_][a-z_0-9]*[ \t]*[(;]"
+	 ;(cons "%[a-z_][a-z_0-9]*[ \t]*[(;]"
+	 ;; SAS macro functions occasionally defined with no arguments
+	 (cons "%[a-z_][a-z_0-9]*[ \t();,]"
 						    font-lock-function-name-face)
 	 (cons "\\<call[ \t]+[a-z_][a-z_0-9]*[ \t]*("
 						    font-lock-function-name-face)
@@ -484,7 +481,7 @@ number."
 		"\\|prob\\([ft]\\|b\\(eta\\|nml\\)\\|chi\\|gam\\|hypr\\|it\\|n\\(egb\\|orm\\)\\)"
 		"\\|ordinal\\|p\\(oisson\\|ut\\)\\|qtr\\|r\\(e\\(peat\\|verse\\)\\|ight\\|ound\\)"
 		"\\|ran\\(bin\\|cau\\|exp\\|g\\(am\\|e\\)\\|k\\|nor\\|poi\\|t\\(bl\\|ri\\)\\|uni\\)"
-		"\\|s\\(aving\\|can\\|econd\\|i\\(gn\\|nh?\\)\\|qrt\\|t\\(d\\(\\|err\\)\\|fips\\|namel?\\)\\|u\\(bstr\\|m\\)\\|ymget\\)"
+                "\\|s\\(aving\\|can\\|econd\\|i\\(gn\\|nh?\\)\\|qrt\\|t\\(d\\(\\|err\\)\\|fips\\|namel?\\)\\|u\\(bstr\\|m\\)\\|ymget\\)"
 		"\\|t\\(anh?\\|i\\(me\\(\\|part\\)\\|nv\\)\\|oday\\|r\\(anslate\\|i\\(gamma\\|m\\)\\|unc\\)\\)"
 		"\\|u\\(niform\\|pcase\\|ss\\)\\|v\\(ar\\|erify\\)"
 		"\\|weekday\\|y\\(ear\\|yq\\)\\|zip\\(fips\\|namel?\\|state\\)"
